@@ -3,22 +3,25 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from copy import deepcopy
+import datetime
 import json
-from typing import Any, Dict, Tuple
-
 import attr
 
+from copy import deepcopy
+from typing import Any, Dict, Tuple
+
+from cassandra.cluster import ResultSet
+
 from swh.model.model import (
-    RevisionType,
     ObjectType,
+    OriginVisitStatus,
     Revision,
+    RevisionType,
     Release,
     Sha1Git,
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS
 
-from ..converters import git_headers_to_db, db_to_git_headers
 from .common import Row
 
 
@@ -29,11 +32,13 @@ def revision_to_db(revision: Revision) -> Dict[str, Any]:
     # non-recursively convert it as a dict but make a deep copy.
     db_revision = deepcopy(attr.asdict(revision, recurse=False))
     metadata = revision.metadata
-    if metadata and "extra_headers" in metadata:
-        db_revision["metadata"]["extra_headers"] = git_headers_to_db(
-            metadata["extra_headers"]
-        )
-    db_revision["metadata"] = json.dumps(db_revision["metadata"])
+    extra_headers = revision.extra_headers
+    if not extra_headers and metadata and "extra_headers" in metadata:
+        extra_headers = db_revision["metadata"].pop("extra_headers")
+    db_revision["metadata"] = json.dumps(
+        dict(db_revision["metadata"]) if db_revision["metadata"] is not None else None
+    )
+    db_revision["extra_headers"] = extra_headers
     db_revision["type"] = db_revision["type"].value
     return db_revision
 
@@ -41,13 +46,16 @@ def revision_to_db(revision: Revision) -> Dict[str, Any]:
 def revision_from_db(db_revision: Row, parents: Tuple[Sha1Git]) -> Revision:
     revision = db_revision._asdict()  # type: ignore
     metadata = json.loads(revision.pop("metadata", None))
-    if metadata and "extra_headers" in metadata:
-        extra_headers = db_to_git_headers(metadata["extra_headers"])
-        metadata["extra_headers"] = extra_headers
+    extra_headers = revision.pop("extra_headers", ())
+    if not extra_headers and metadata and "extra_headers" in metadata:
+        extra_headers = metadata.pop("extra_headers")
+    if extra_headers is None:
+        extra_headers = ()
     return Revision(
         parents=parents,
         type=RevisionType(revision.pop("type")),
         metadata=metadata,
+        extra_headers=extra_headers,
         **revision,
     )
 
@@ -71,3 +79,18 @@ def row_to_content_hashes(row: Row) -> Dict[str, bytes]:
     for algo in DEFAULT_ALGORITHMS:
         hashes[algo] = getattr(row, algo)
     return hashes
+
+
+def row_to_visit_status(row: ResultSet) -> OriginVisitStatus:
+    """Format a row representing a visit_status to an actual dict representing an
+    OriginVisitStatus.
+
+    """
+    return OriginVisitStatus.from_dict(
+        {
+            **row._asdict(),
+            "origin": row.origin,
+            "date": row.date.replace(tzinfo=datetime.timezone.utc),
+            "metadata": (json.loads(row.metadata) if row.metadata else None),
+        }
+    )
