@@ -18,6 +18,7 @@ from typing import (
     Optional,
     Tuple,
     TypeVar,
+    Union,
 )
 
 from cassandra import CoordinationFailure
@@ -45,6 +46,7 @@ from swh.model.model import (
 
 from .common import Row, TOKEN_BEGIN, TOKEN_END, hash_url
 from .schema import CREATE_TABLES_QUERIES, HASH_ALGORITHMS
+from .. import extrinsic_metadata
 
 
 logger = logging.getLogger(__name__)
@@ -282,9 +284,7 @@ class CqlRunner:
     # 'content_by_*' tables
     ##########################
 
-    @_prepared_statement(
-        "SELECT sha1_git FROM content_by_sha1_git " "WHERE sha1_git IN ?"
-    )
+    @_prepared_statement("SELECT sha1_git FROM content_by_sha1_git WHERE sha1_git IN ?")
     def content_missing_by_sha1_git(
         self, ids: List[bytes], *, statement
     ) -> List[bytes]:
@@ -425,6 +425,7 @@ class CqlRunner:
         "committer",
         "synthetic",
         "metadata",
+        "extra_headers",
     ]
 
     @_prepared_exists_statement("revision")
@@ -533,7 +534,7 @@ class CqlRunner:
             statement, [entry[key] for key in self._directory_entry_keys]
         )
 
-    @_prepared_statement("SELECT * FROM directory_entry " "WHERE directory_id IN ?")
+    @_prepared_statement("SELECT * FROM directory_entry WHERE directory_id IN ?")
     def directory_entry_get(self, directory_ids, *, statement) -> ResultSet:
         return self._execute_with_retries(statement, [directory_ids])
 
@@ -552,7 +553,7 @@ class CqlRunner:
         self._execute_with_retries(statement, [snapshot_id])
         self._increment_counter("snapshot", 1)
 
-    @_prepared_statement("SELECT * FROM snapshot " "WHERE id = ?")
+    @_prepared_statement("SELECT * FROM snapshot WHERE id = ?")
     def snapshot_get(self, snapshot_id: Sha1Git, *, statement) -> ResultSet:
         return self._execute_with_retries(statement, [snapshot_id])
 
@@ -581,7 +582,7 @@ class CqlRunner:
         return self._execute_with_retries(statement, [snapshot_id])
 
     @_prepared_statement(
-        "SELECT * FROM snapshot_branch " "WHERE snapshot_id = ? AND name >= ?" "LIMIT ?"
+        "SELECT * FROM snapshot_branch WHERE snapshot_id = ? AND name >= ? LIMIT ?"
     )
     def snapshot_branch_get(
         self, snapshot_id: Sha1Git, from_: bytes, limit: int, *, statement
@@ -627,7 +628,7 @@ class CqlRunner:
         return rows[0].next_visit_id
 
     @_prepared_statement(
-        "UPDATE origin SET next_visit_id=? " "WHERE sha1 = ? IF next_visit_id=?"
+        "UPDATE origin SET next_visit_id=? WHERE sha1 = ? IF next_visit_id=?"
     )
     def origin_generate_unique_visit_id(self, origin_url: str, *, statement) -> int:
         origin_sha1 = hash_url(origin_url)
@@ -658,36 +659,105 @@ class CqlRunner:
         "visit",
         "type",
         "date",
-        "status",
-        "metadata",
-        "snapshot",
     ]
-    _origin_visit_update_keys = ["type", "date", "status", "metadata", "snapshot"]
 
-    @_prepared_statement("SELECT * FROM origin_visit " "WHERE origin = ? AND visit > ?")
-    def _origin_visit_get_no_limit(
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? AND visit > ? "
+        "ORDER BY visit ASC"
+    )
+    def _origin_visit_get_pagination_asc_no_limit(
         self, origin_url: str, last_visit: int, *, statement
     ) -> ResultSet:
         return self._execute_with_retries(statement, [origin_url, last_visit])
 
     @_prepared_statement(
-        "SELECT * FROM origin_visit " "WHERE origin = ? AND visit > ? LIMIT ?"
+        "SELECT * FROM origin_visit WHERE origin = ? AND visit > ? "
+        "ORDER BY visit ASC "
+        "LIMIT ?"
     )
-    def _origin_visit_get_limit(
+    def _origin_visit_get_pagination_asc_limit(
         self, origin_url: str, last_visit: int, limit: int, *, statement
     ) -> ResultSet:
         return self._execute_with_retries(statement, [origin_url, last_visit, limit])
 
-    def origin_visit_get(
-        self, origin_url: str, last_visit: Optional[int], limit: Optional[int]
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? AND visit < ? "
+        "ORDER BY visit DESC"
+    )
+    def _origin_visit_get_pagination_desc_no_limit(
+        self, origin_url: str, last_visit: int, *, statement
     ) -> ResultSet:
-        if last_visit is None:
-            last_visit = -1
+        return self._execute_with_retries(statement, [origin_url, last_visit])
 
-        if limit is None:
-            return self._origin_visit_get_no_limit(origin_url, last_visit)
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? AND visit < ? "
+        "ORDER BY visit DESC "
+        "LIMIT ?"
+    )
+    def _origin_visit_get_pagination_desc_limit(
+        self, origin_url: str, last_visit: int, limit: int, *, statement
+    ) -> ResultSet:
+        return self._execute_with_retries(statement, [origin_url, last_visit, limit])
+
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? ORDER BY visit ASC LIMIT ?"
+    )
+    def _origin_visit_get_no_pagination_asc_limit(
+        self, origin_url: str, limit: int, *, statement
+    ) -> ResultSet:
+        return self._execute_with_retries(statement, [origin_url, limit])
+
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? ORDER BY visit ASC "
+    )
+    def _origin_visit_get_no_pagination_asc_no_limit(
+        self, origin_url: str, *, statement
+    ) -> ResultSet:
+        return self._execute_with_retries(statement, [origin_url])
+
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? ORDER BY visit DESC"
+    )
+    def _origin_visit_get_no_pagination_desc_no_limit(
+        self, origin_url: str, *, statement
+    ) -> ResultSet:
+        return self._execute_with_retries(statement, [origin_url])
+
+    @_prepared_statement(
+        "SELECT * FROM origin_visit WHERE origin = ? ORDER BY visit DESC LIMIT ?"
+    )
+    def _origin_visit_get_no_pagination_desc_limit(
+        self, origin_url: str, limit: int, *, statement
+    ) -> ResultSet:
+        return self._execute_with_retries(statement, [origin_url, limit])
+
+    def origin_visit_get(
+        self,
+        origin_url: str,
+        last_visit: Optional[int],
+        limit: Optional[int],
+        order: str = "asc",
+    ) -> ResultSet:
+        order = order.lower()
+        assert order in ["asc", "desc"]
+
+        args: List[Any] = [origin_url]
+
+        if last_visit is not None:
+            page_name = "pagination"
+            args.append(last_visit)
         else:
-            return self._origin_visit_get_limit(origin_url, last_visit, limit)
+            page_name = "no_pagination"
+
+        if limit is not None:
+            limit_name = "limit"
+            args.append(limit)
+        else:
+            limit_name = "no_limit"
+
+        method_name = f"_origin_visit_get_{page_name}_{order}_{limit_name}"
+        origin_visit_get_method = getattr(self, method_name)
+        return origin_visit_get_method(*args)
 
     @_prepared_insert_statement("origin_visit", _origin_visit_keys)
     def origin_visit_add_one(self, visit: OriginVisit, *, statement) -> None:
@@ -709,62 +779,40 @@ class CqlRunner:
         assert self._origin_visit_status_keys[-1] == "metadata"
         keys = self._origin_visit_status_keys
 
-        metadata = json.dumps(visit_update.metadata)
+        metadata = json.dumps(
+            dict(visit_update.metadata) if visit_update.metadata is not None else None
+        )
         self._execute_with_retries(
             statement, [getattr(visit_update, key) for key in keys[:-1]] + [metadata]
         )
 
-    def _format_origin_visit_status_row(
-        self, visit_status: ResultSet
-    ) -> Dict[str, Any]:
-        """Format a row visit_status into an origin_visit_status dict
+    def origin_visit_status_get_latest(self, origin: str, visit: int,) -> Optional[Row]:
+        """Given an origin visit id, return its latest origin_visit_status
 
-        """
-        return {
-            **visit_status._asdict(),
-            "origin": visit_status.origin,
-            "date": visit_status.date.replace(tzinfo=datetime.timezone.utc),
-            "metadata": (
-                json.loads(visit_status.metadata) if visit_status.metadata else None
-            ),
-        }
+         """
+        rows = self.origin_visit_status_get(origin, visit)
+        return rows[0] if rows else None
 
     @_prepared_statement(
         "SELECT * FROM origin_visit_status "
         "WHERE origin = ? AND visit = ? "
-        "ORDER BY date DESC "
-        "LIMIT 1"
+        "ORDER BY date DESC"
     )
-    def origin_visit_status_get_latest(
-        self, origin: str, visit: int, *, statement
-    ) -> Optional[Dict[str, Any]]:
-        """Given an origin visit id, return its latest origin_visit_status
+    def origin_visit_status_get(
+        self,
+        origin: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
+        *,
+        statement,
+    ) -> List[Row]:
+        """Return all origin visit statuses for a given visit
 
         """
-        rows = list(self._execute_with_retries(statement, [origin, visit]))
-        if rows:
-            return self._format_origin_visit_status_row(rows[0])
-        else:
-            return None
+        return list(self._execute_with_retries(statement, [origin, visit]))
 
-    @_prepared_statement(
-        "UPDATE origin_visit SET "
-        + ", ".join("%s = ?" % key for key in _origin_visit_update_keys)
-        + " WHERE origin = ? AND visit = ?"
-    )
-    def origin_visit_upsert(self, visit: OriginVisit, *, statement) -> None:
-        args: List[Any] = []
-        for column in self._origin_visit_update_keys:
-            if column == "metadata":
-                args.append(json.dumps(visit.metadata))
-            else:
-                args.append(getattr(visit, column))
-
-        self._execute_with_retries(statement, args + [visit.origin, visit.visit])
-        # TODO:  check if there is already one
-        self._increment_counter("origin_visit", 1)
-
-    @_prepared_statement("SELECT * FROM origin_visit " "WHERE origin = ? AND visit = ?")
+    @_prepared_statement("SELECT * FROM origin_visit WHERE origin = ? AND visit = ?")
     def origin_visit_get_one(
         self, origin_url: str, visit_id: int, *, statement
     ) -> Optional[Row]:
@@ -775,7 +823,7 @@ class CqlRunner:
         else:
             return None
 
-    @_prepared_statement("SELECT * FROM origin_visit " "WHERE origin = ?")
+    @_prepared_statement("SELECT * FROM origin_visit WHERE origin = ?")
     def origin_visit_get_all(self, origin_url: str, *, statement) -> ResultSet:
         return self._execute_with_retries(statement, [origin_url])
 
@@ -823,12 +871,13 @@ class CqlRunner:
     def metadata_fetcher_get(self, name, version, *, statement) -> Optional[Row]:
         return next(iter(self._execute_with_retries(statement, [name, version])), None)
 
-    ##########################
-    # 'origin_metadata' table
-    ##########################
+    #########################
+    # 'object_metadata' table
+    #########################
 
-    _origin_metadata_keys = [
-        "origin",
+    _object_metadata_keys = [
+        "type",
+        "id",
         "authority_type",
         "authority_url",
         "discovery_date",
@@ -836,12 +885,23 @@ class CqlRunner:
         "fetcher_version",
         "format",
         "metadata",
+        "origin",
+        "visit",
+        "snapshot",
+        "release",
+        "revision",
+        "path",
+        "directory",
     ]
 
-    @_prepared_insert_statement("origin_metadata", _origin_metadata_keys)
-    def origin_metadata_add(
+    @_prepared_statement(
+        f"INSERT INTO object_metadata ({', '.join(_object_metadata_keys)}) "
+        f"VALUES ({', '.join('?' for _ in _object_metadata_keys)})"
+    )
+    def object_metadata_add(
         self,
-        origin,
+        object_type: str,
+        id: str,
         authority_type,
         authority_url,
         discovery_date,
@@ -849,55 +909,65 @@ class CqlRunner:
         fetcher_version,
         format,
         metadata,
+        context: Dict[str, Union[str, bytes, int]],
+        *,
+        statement,
+    ):
+        params = [
+            object_type,
+            id,
+            authority_type,
+            authority_url,
+            discovery_date,
+            fetcher_name,
+            fetcher_version,
+            format,
+            metadata,
+        ]
+
+        params.extend(
+            context.get(key) for key in extrinsic_metadata.CONTEXT_KEYS[object_type]
+        )
+
+        return self._execute_with_retries(statement, params,)
+
+    @_prepared_statement(
+        "SELECT * from object_metadata "
+        "WHERE id=? AND authority_url=? AND discovery_date>? AND authority_type=?"
+    )
+    def object_metadata_get_after_date(
+        self,
+        id: str,
+        authority_type: str,
+        authority_url: str,
+        after: datetime.datetime,
         *,
         statement,
     ):
         return self._execute_with_retries(
-            statement,
-            [
-                origin,
-                authority_type,
-                authority_url,
-                discovery_date,
-                fetcher_name,
-                fetcher_version,
-                format,
-                metadata,
-            ],
+            statement, [id, authority_url, after, authority_type]
         )
 
     @_prepared_statement(
-        "SELECT * from origin_metadata "
-        "WHERE origin=? AND authority_url=? AND discovery_date>? "
-        "AND authority_type=?"
-    )
-    def origin_metadata_get_after_date(
-        self, origin, authority_type, authority_url, after, *, statement
-    ):
-        return self._execute_with_retries(
-            statement, [origin, authority_url, after, authority_type]
-        )
-
-    @_prepared_statement(
-        "SELECT * from origin_metadata "
-        "WHERE origin=? AND authority_type=? AND authority_url=? "
+        "SELECT * from object_metadata "
+        "WHERE id=? AND authority_type=? AND authority_url=? "
         "AND (discovery_date, fetcher_name, fetcher_version) > (?, ?, ?)"
     )
-    def origin_metadata_get_after_date_and_fetcher(
+    def object_metadata_get_after_date_and_fetcher(
         self,
-        origin,
-        authority_type,
-        authority_url,
-        after_date,
-        after_fetcher_name,
-        after_fetcher_version,
+        id: str,
+        authority_type: str,
+        authority_url: str,
+        after_date: datetime.datetime,
+        after_fetcher_name: str,
+        after_fetcher_version: str,
         *,
         statement,
     ):
         return self._execute_with_retries(
             statement,
             [
-                origin,
+                id,
                 authority_type,
                 authority_url,
                 after_date,
@@ -907,14 +977,14 @@ class CqlRunner:
         )
 
     @_prepared_statement(
-        "SELECT * from origin_metadata "
-        "WHERE origin=? AND authority_url=? AND authority_type=?"
+        "SELECT * from object_metadata "
+        "WHERE id=? AND authority_url=? AND authority_type=?"
     )
-    def origin_metadata_get(
-        self, origin, authority_type, authority_url, *, statement
+    def object_metadata_get(
+        self, id: str, authority_type: str, authority_url: str, *, statement
     ) -> Iterable[Row]:
         return self._execute_with_retries(
-            statement, [origin, authority_url, authority_type]
+            statement, [id, authority_url, authority_type]
         )
 
     ##########################
@@ -926,7 +996,7 @@ class CqlRunner:
         self._execute_with_retries(statement, [])
 
     @_prepared_statement(
-        "SELECT object_type, count FROM object_count " "WHERE partition_key=0"
+        "SELECT object_type, count FROM object_count WHERE partition_key=0"
     )
     def stat_counters(self, *, statement) -> ResultSet:
         return self._execute_with_retries(statement, [])
