@@ -30,29 +30,34 @@ $$
 $$
 ;
 
+
 CREATE OR REPLACE AGGREGATE ascii_bins_count ( ascii )
 SFUNC ascii_bins_count_sfunc
 STYPE tuple<int, map<ascii, int>>
 INITCOND (0, {})
 ;
 
+
 CREATE TYPE IF NOT EXISTS microtimestamp (
     seconds             bigint,
     microseconds        int
 );
 
+
 CREATE TYPE IF NOT EXISTS microtimestamp_with_timezone (
     timestamp           frozen<microtimestamp>,
     offset              smallint,
     negative_utc        boolean
 );
 
+
 CREATE TYPE IF NOT EXISTS person (
     fullname    blob,
     name        blob,
     email       blob
 );
 
+
 CREATE TABLE IF NOT EXISTS content (
     sha1          blob,
     sha1_git      blob,
@@ -65,6 +70,7 @@ CREATE TABLE IF NOT EXISTS content (
     PRIMARY KEY ((sha1, sha1_git, sha256, blake2s256))
 );
 
+
 CREATE TABLE IF NOT EXISTS skipped_content (
     sha1          blob,
     sha1_git      blob,
@@ -79,6 +85,7 @@ CREATE TABLE IF NOT EXISTS skipped_content (
     PRIMARY KEY ((sha1, sha1_git, sha256, blake2s256))
 );
 
+
 CREATE TABLE IF NOT EXISTS revision (
     id                              blob PRIMARY KEY,
     date                            microtimestamp_with_timezone,
@@ -90,11 +97,13 @@ CREATE TABLE IF NOT EXISTS revision (
     committer                       person,
     synthetic                       boolean,
         -- true iff revision has been created by Software Heritage
-    metadata                        text
-        -- extra metadata as JSON(tarball checksums,
-        -- extra commit information, etc...)
+    metadata                        text,
+        -- extra metadata as JSON(tarball checksums, etc...)
+    extra_headers                   frozen<list <list<blob>> >
+        -- extra commit information as (tuple(key, value), ...)
 );
 
+
 CREATE TABLE IF NOT EXISTS revision_parent (
     id                     blob,
     parent_rank                     int,
@@ -103,6 +112,7 @@ CREATE TABLE IF NOT EXISTS revision_parent (
     PRIMARY KEY ((id), parent_rank)
 );
 
+
 CREATE TABLE IF NOT EXISTS release
 (
     id                              blob PRIMARY KEY,
@@ -116,10 +126,12 @@ CREATE TABLE IF NOT EXISTS release
         -- true iff release has been created by Software Heritage
 );
 
+
 CREATE TABLE IF NOT EXISTS directory (
     id              blob PRIMARY KEY,
 );
 
+
 CREATE TABLE IF NOT EXISTS directory_entry (
     directory_id    blob,
     name            blob,  -- path name, relative to containing dir
@@ -129,10 +141,12 @@ CREATE TABLE IF NOT EXISTS directory_entry (
     PRIMARY KEY ((directory_id), name)
 );
 
+
 CREATE TABLE IF NOT EXISTS snapshot (
     id              blob PRIMARY KEY,
 );
 
+
 -- For a given snapshot_id, branches are sorted by their name,
 -- allowing easy pagination.
 CREATE TABLE IF NOT EXISTS snapshot_branch (
@@ -143,17 +157,16 @@ CREATE TABLE IF NOT EXISTS snapshot_branch (
     PRIMARY KEY ((snapshot_id), name)
 );
 
+
 CREATE TABLE IF NOT EXISTS origin_visit (
     origin          text,
     visit           bigint,
     date            timestamp,
     type            text,
-    status          ascii,
-    metadata        text,
-    snapshot        blob,
     PRIMARY KEY ((origin), visit)
 );
 
+
 CREATE TABLE IF NOT EXISTS origin_visit_status (
     origin          text,
     visit           bigint,
@@ -164,6 +177,7 @@ CREATE TABLE IF NOT EXISTS origin_visit_status (
     PRIMARY KEY ((origin), visit, date)
 );
 
+
 CREATE TABLE IF NOT EXISTS origin (
     sha1            blob PRIMARY KEY,
     url             text,
@@ -191,20 +205,32 @@ CREATE TABLE IF NOT EXISTS metadata_fetcher (
 );
 
 
-CREATE TABLE IF NOT EXISTS origin_metadata (
-    origin          text,
+CREATE TABLE IF NOT EXISTS object_metadata (
+    type            text,
+    id              text,
+
+    -- metadata source
     authority_type  text,
     authority_url   text,
     discovery_date  timestamp,
     fetcher_name    ascii,
     fetcher_version ascii,
+
+    -- metadata itself
     format          ascii,
     metadata        blob,
-    PRIMARY KEY ((origin), authority_type, authority_url, discovery_date,
-                           fetcher_name, fetcher_version),
-    -- for now, authority_url could be in the partition key; but leaving
-    -- in the partition key allows listing authorities with metadata on an
-    -- origin if we ever need to do it.
+
+    -- context
+    origin          text,
+    visit           bigint,
+    snapshot        text,
+    release         text,
+    revision        text,
+    path            blob,
+    directory       text,
+
+    PRIMARY KEY ((id), authority_type, authority_url, discovery_date,
+                       fetcher_name, fetcher_version)
 );
 
 
@@ -215,7 +241,7 @@ CREATE TABLE IF NOT EXISTS object_count (
     PRIMARY KEY ((partition_key), object_type)
 );
 """.split(
-    "\n\n"
+    "\n\n\n"
 )
 
 CONTENT_INDEX_TEMPLATE = """
@@ -236,7 +262,7 @@ CREATE TABLE IF NOT EXISTS skipped_content_by_{main_algo} (
 TABLES = (
     "skipped_content content revision revision_parent release "
     "directory directory_entry snapshot snapshot_branch "
-    "origin_visit origin origin_metadata object_count "
+    "origin_visit origin object_metadata object_count "
     "origin_visit_status metadata_authority "
     "metadata_fetcher"
 ).split()
