@@ -8,9 +8,10 @@ import itertools
 import json
 import random
 import re
-from typing import Any, Dict, List, Iterable, Optional
+from typing import Any, Dict, List, Iterable, Optional, Union
 
 import attr
+from deprecated import deprecated
 
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
 from swh.model.model import (
@@ -28,16 +29,17 @@ from swh.model.model import (
 from swh.model.hashutil import DEFAULT_ALGORITHMS
 from swh.storage.objstorage import ObjStorage
 from swh.storage.writer import JournalWriter
-from swh.storage.validate import convert_validation_exceptions
 from swh.storage.utils import now
 
 from ..exc import StorageArgumentException, HashCollision
+from ..extrinsic_metadata import check_extrinsic_metadata_context, CONTEXT_KEYS
 from .common import TOKEN_BEGIN, TOKEN_END
 from .converters import (
     revision_to_db,
     revision_from_db,
     release_to_db,
     release_from_db,
+    row_to_visit_status,
 )
 from .cql import CqlRunner
 from .schema import HASH_ALGORITHMS
@@ -511,16 +513,20 @@ class CassandraStorage:
         return self._cql_runner.revision_get_random().id
 
     def release_add(self, releases: Iterable[Release]) -> Dict:
-        missing = self.release_missing([rel.id for rel in releases])
-        releases = [rel for rel in releases if rel.id in missing]
+        to_add = []
+        for rel in releases:
+            if rel not in to_add:
+                to_add.append(rel)
+        missing = set(self.release_missing([rel.id for rel in to_add]))
+        to_add = [rel for rel in to_add if rel.id in missing]
 
-        self.journal_writer.release_add(releases)
+        self.journal_writer.release_add(to_add)
 
-        for release in releases:
+        for release in to_add:
             if release:
                 self._cql_runner.release_add_one(release_to_db(release))
 
-        return {"release:add": len(missing)}
+        return {"release:add": len(to_add)}
 
     def release_missing(self, releases):
         return self._cql_runner.release_missing(releases)
@@ -583,17 +589,6 @@ class CassandraStorage:
 
         return self.snapshot_get(visit["snapshot"])
 
-    def snapshot_get_latest(self, origin, allowed_statuses=None):
-        visit = self.origin_visit_get_latest(
-            origin, allowed_statuses=allowed_statuses, require_snapshot=True
-        )
-
-        if visit:
-            assert visit["snapshot"]
-            if self._cql_runner.snapshot_missing([visit["snapshot"]]):
-                raise StorageArgumentException("Visit references unknown snapshot")
-            return self.snapshot_get_branches(visit["snapshot"])
-
     def snapshot_count_branches(self, snapshot_id):
         if self._cql_runner.snapshot_missing([snapshot_id]):
             # Makes sure we don't fetch branches for a snapshot that is
@@ -774,13 +769,19 @@ class CassandraStorage:
 
         return [{"url": orig.url,} for orig in origins[offset : offset + limit]]
 
-    def origin_add(self, origins: Iterable[Origin]) -> List[Dict]:
-        results = []
-        for origin in origins:
-            self.origin_add_one(origin)
-            results.append(origin.to_dict())
-        return results
+    def origin_add(self, origins: Iterable[Origin]) -> Dict[str, int]:
+        known_origins = [
+            Origin.from_dict(d)
+            for d in self.origin_get([origin.to_dict() for origin in origins])
+            if d is not None
+        ]
+        to_add = [origin for origin in origins if origin not in known_origins]
+        self.journal_writer.origin_add(to_add)
+        for origin in to_add:
+            self._cql_runner.origin_add_one(origin)
+        return {"origin:add": len(to_add)}
 
+    @deprecated("Use origin_add([origin]) instead")
     def origin_add_one(self, origin: Origin) -> str:
         known_origin = self.origin_get_one(origin.to_dict())
 
@@ -813,11 +814,15 @@ class CassandraStorage:
             self._cql_runner.origin_visit_add_one(visit)
             assert visit.visit is not None
             all_visits.append(visit)
-
-            visit_status_dict = visit.to_dict()
-            visit_status_dict.pop("type")
-            visit_status = OriginVisitStatus.from_dict(visit_status_dict)
-            self._origin_visit_status_add(visit_status)
+            self._origin_visit_status_add(
+                OriginVisitStatus(
+                    origin=visit.origin,
+                    visit=visit.visit,
+                    date=visit.date,
+                    status="created",
+                    snapshot=None,
+                )
+            )
 
         return all_visits
 
@@ -835,83 +840,30 @@ class CassandraStorage:
             if not origin_url:
                 raise StorageArgumentException(f"Unknown origin {visit_status.origin}")
 
-        self.journal_writer.origin_visit_status_add(visit_statuses)
         for visit_status in visit_statuses:
             self._origin_visit_status_add(visit_status)
 
-    def origin_visit_update(
-        self,
-        origin: str,
-        visit_id: int,
-        status: str,
-        metadata: Optional[Dict] = None,
-        snapshot: Optional[bytes] = None,
-        date: Optional[datetime.datetime] = None,
-    ):
-        origin_url = origin  # TODO: rename the argument
-
-        # Get the existing data of the visit
-        visit_ = self.origin_visit_get_by(origin_url, visit_id)
-        if not visit_:
-            raise StorageArgumentException("This origin visit does not exist.")
-        with convert_validation_exceptions():
-            visit = OriginVisit.from_dict(visit_)
-
-        updates: Dict[str, Any] = {"status": status}
-        if metadata and metadata != visit.metadata:
-            updates["metadata"] = metadata
-        if snapshot and snapshot != visit.snapshot:
-            updates["snapshot"] = snapshot
-
-        with convert_validation_exceptions():
-            visit = attr.evolve(visit, **updates)
-
-        self.journal_writer.origin_visit_update([visit])
-
-        last_visit_update = self._origin_visit_get_updated(visit.origin, visit.visit)
-        assert last_visit_update is not None
-
-        with convert_validation_exceptions():
-            visit_status = OriginVisitStatus(
-                origin=origin_url,
-                visit=visit_id,
-                date=date or now(),
-                status=status,
-                snapshot=snapshot or last_visit_update["snapshot"],
-                metadata=metadata or last_visit_update["metadata"],
-            )
-        self._origin_visit_status_add(visit_status)
-
-    def _origin_visit_merge(
-        self, visit: Dict[str, Any], visit_status: Dict[str, Any]
-    ) -> Dict[str, Any]:
-        """Merge origin_visit and visit_status together.
-
-        """
-        return OriginVisit.from_dict(
-            {
-                # default to the values in visit
-                **visit,
-                # override with the last update
-                **visit_status,
-                # visit['origin'] is the URL (via a join), while
-                # visit_status['origin'] is only an id.
-                "origin": visit["origin"],
-                # but keep the date of the creation of the origin visit
-                "date": visit["date"],
-            }
-        ).to_dict()
-
     def _origin_visit_apply_last_status(self, visit: Dict[str, Any]) -> Dict[str, Any]:
         """Retrieve the latest visit status information for the origin visit.
         Then merge it with the visit and return it.
 
         """
-        visit_status = self._cql_runner.origin_visit_status_get_latest(
+        row = self._cql_runner.origin_visit_status_get_latest(
             visit["origin"], visit["visit"]
         )
-        assert visit_status is not None
-        return self._origin_visit_merge(visit, visit_status)
+        assert row is not None
+        visit_status = row_to_visit_status(row)
+        return {
+            # default to the values in visit
+            **visit,
+            # override with the last update
+            **visit_status.to_dict(),
+            # visit['origin'] is the URL (via a join), while
+            # visit_status['origin'] is only an id.
+            "origin": visit["origin"],
+            # but keep the date of the creation of the origin visit
+            "date": visit["date"],
+        }
 
     def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
         """Retrieve origin visit and latest origin visit status and merge them
@@ -923,39 +875,23 @@ class CassandraStorage:
         visit = self._format_origin_visit_row(row_visit)
         return self._origin_visit_apply_last_status(visit)
 
-    def origin_visit_upsert(self, visits: Iterable[OriginVisit]) -> None:
-        for visit in visits:
-            if visit.visit is None:
-                raise StorageArgumentException(f"Missing visit id for visit {visit}")
-
-        self.journal_writer.origin_visit_upsert(visits)
-        for visit in visits:
-            assert visit.visit is not None
-            self._cql_runner.origin_visit_upsert(visit)
-            with convert_validation_exceptions():
-                visit_status = OriginVisitStatus(
-                    origin=visit.origin,
-                    visit=visit.visit,
-                    date=now(),
-                    status=visit.status,
-                    snapshot=visit.snapshot,
-                    metadata=visit.metadata,
-                )
-            self._origin_visit_status_add(visit_status)
-
     @staticmethod
     def _format_origin_visit_row(visit):
         return {
             **visit._asdict(),
             "origin": visit.origin,
             "date": visit.date.replace(tzinfo=datetime.timezone.utc),
-            "metadata": (json.loads(visit.metadata) if visit.metadata else None),
         }
 
     def origin_visit_get(
-        self, origin: str, last_visit: Optional[int] = None, limit: Optional[int] = None
+        self,
+        origin: str,
+        last_visit: Optional[int] = None,
+        limit: Optional[int] = None,
+        order: str = "asc",
     ) -> Iterable[Dict[str, Any]]:
-        rows = self._cql_runner.origin_visit_get(origin, last_visit, limit)
+        rows = self._cql_runner.origin_visit_get(origin, last_visit, limit, order)
+
         for row in rows:
             visit = self._format_origin_visit_row(row)
             yield self._origin_visit_apply_last_status(visit)
@@ -988,6 +924,7 @@ class CassandraStorage:
     def origin_visit_get_latest(
         self,
         origin: str,
+        type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[Dict[str, Any]]:
@@ -997,6 +934,8 @@ class CassandraStorage:
         for row in rows:
             visit = self._format_origin_visit_row(row)
             updated_visit = self._origin_visit_apply_last_status(visit)
+            if type is not None and updated_visit["type"] != type:
+                continue
             if allowed_statuses and updated_visit["status"] not in allowed_statuses:
                 continue
             if require_snapshot and updated_visit["snapshot"] is None:
@@ -1013,6 +952,25 @@ class CassandraStorage:
 
         return latest_visit
 
+    def origin_visit_status_get_latest(
+        self,
+        origin_url: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
+    ) -> Optional[OriginVisitStatus]:
+        rows = self._cql_runner.origin_visit_status_get(
+            origin_url, visit, allowed_statuses, require_snapshot
+        )
+        # filtering is done python side as we cannot do it server side
+        if allowed_statuses:
+            rows = [row for row in rows if row.status in allowed_statuses]
+        if require_snapshot:
+            rows = [row for row in rows if row.snapshot is not None]
+        if not rows:
+            return None
+        return row_to_visit_status(rows[0])
+
     def origin_visit_get_random(self, type: str) -> Optional[Dict[str, Any]]:
         back_in_the_day = now() - datetime.timedelta(weeks=12)  # 3 months back
 
@@ -1051,6 +1009,39 @@ class CassandraStorage:
     def refresh_stat_counters(self):
         pass
 
+    def content_metadata_add(
+        self,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+    ) -> None:
+        self._object_metadata_add(
+            "content",
+            id,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            context,
+        )
+
+    def content_metadata_get(
+        self,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
+        return self._object_metadata_get(
+            "content", id, authority, after, page_token, limit,
+        )
+
     def origin_metadata_add(
         self,
         origin_url: str,
@@ -1062,26 +1053,21 @@ class CassandraStorage:
     ) -> None:
         if not isinstance(origin_url, str):
             raise StorageArgumentException(
-                "origin_id must be str, not %r" % (origin_url,)
+                "origin_url must be str, not %r" % (origin_url,)
             )
-        if not self._cql_runner.metadata_authority_get(**authority):
-            raise StorageArgumentException(f"Unknown authority {authority}")
-        if not self._cql_runner.metadata_fetcher_get(**fetcher):
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
 
-        try:
-            self._cql_runner.origin_metadata_add(
-                origin_url,
-                authority["type"],
-                authority["url"],
-                discovery_date,
-                fetcher["name"],
-                fetcher["version"],
-                format,
-                metadata,
-            )
-        except TypeError as e:
-            raise StorageArgumentException(*e.args)
+        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
+
+        self._object_metadata_add(
+            "origin",
+            origin_url,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            context,
+        )
 
     def origin_metadata_get(
         self,
@@ -1094,6 +1080,57 @@ class CassandraStorage:
         if not isinstance(origin_url, str):
             raise TypeError("origin_url must be str, not %r" % (origin_url,))
 
+        res = self._object_metadata_get(
+            "origin", origin_url, authority, after, page_token, limit
+        )
+        for result in res["results"]:
+            result["origin_url"] = result.pop("id")
+
+        return res
+
+    def _object_metadata_add(
+        self,
+        object_type: str,
+        id: str,
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+        context: Dict[str, Union[str, bytes, int]],
+    ) -> None:
+        check_extrinsic_metadata_context(object_type, context)
+
+        if not self._cql_runner.metadata_authority_get(**authority):
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        if not self._cql_runner.metadata_fetcher_get(**fetcher):
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
+
+        try:
+            self._cql_runner.object_metadata_add(
+                object_type,
+                id,
+                authority["type"],
+                authority["url"],
+                discovery_date,
+                fetcher["name"],
+                fetcher["version"],
+                format,
+                metadata,
+                context,
+            )
+        except TypeError as e:
+            raise StorageArgumentException(*e.args)
+
+    def _object_metadata_get(
+        self,
+        object_type: str,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
         if page_token is not None:
             (after_date, after_fetcher_name, after_fetcher_url) = msgpack_loads(
                 page_token
@@ -1102,8 +1139,8 @@ class CassandraStorage:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._cql_runner.origin_metadata_get_after_date_and_fetcher(
-                origin_url,
+            entries = self._cql_runner.object_metadata_get_after_date_and_fetcher(
+                id,
                 authority["type"],
                 authority["url"],
                 after_date,
@@ -1111,12 +1148,12 @@ class CassandraStorage:
                 after_fetcher_url,
             )
         elif after is not None:
-            entries = self._cql_runner.origin_metadata_get_after_date(
-                origin_url, authority["type"], authority["url"], after
+            entries = self._cql_runner.object_metadata_get_after_date(
+                id, authority["type"], authority["url"], after
             )
         else:
-            entries = self._cql_runner.origin_metadata_get(
-                origin_url, authority["type"], authority["url"]
+            entries = self._cql_runner.object_metadata_get(
+                id, authority["type"], authority["url"]
             )
 
         if limit:
@@ -1125,22 +1162,31 @@ class CassandraStorage:
         results = []
         for entry in entries:
             discovery_date = entry.discovery_date.replace(tzinfo=datetime.timezone.utc)
-            results.append(
-                {
-                    "origin_url": entry.origin,
-                    "authority": {
-                        "type": entry.authority_type,
-                        "url": entry.authority_url,
-                    },
-                    "fetcher": {
-                        "name": entry.fetcher_name,
-                        "version": entry.fetcher_version,
-                    },
-                    "discovery_date": discovery_date,
-                    "format": entry.format,
-                    "metadata": entry.metadata,
-                }
-            )
+
+            result = {
+                "id": entry.id,
+                "authority": {
+                    "type": entry.authority_type,
+                    "url": entry.authority_url,
+                },
+                "fetcher": {
+                    "name": entry.fetcher_name,
+                    "version": entry.fetcher_version,
+                },
+                "discovery_date": discovery_date,
+                "format": entry.format,
+                "metadata": entry.metadata,
+            }
+
+            if CONTEXT_KEYS[object_type]:
+                context = {}
+                for key in CONTEXT_KEYS[object_type]:
+                    value = getattr(entry, key)
+                    if value is not None:
+                        context[key] = value
+                result["context"] = context
+
+            results.append(result)
 
         if len(results) > limit:
             results.pop()
