@@ -6,14 +6,18 @@
 import datetime
 import random
 import select
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 from swh.core.db import BaseDb
-from swh.core.db.db_utils import stored_procedure, jsonize
+from swh.core.db.db_utils import stored_procedure, jsonize as _jsonize
 from swh.core.db.db_utils import execute_values_generator
 from swh.model.model import OriginVisit, OriginVisitStatus, SHA1_SIZE
 
 
+def jsonize(d):
+    return _jsonize(dict(d) if d is not None else None)
+
+
 class Db(BaseDb):
     """Proxy to the SWH DB, with wrappers around stored procedures
 
@@ -423,13 +427,13 @@ class Db(BaseDb):
         "committer_email",
         "metadata",
         "synthetic",
+        "extra_headers",
     ]
 
     revision_get_cols = revision_add_cols + ["parents"]
 
     def origin_visit_add(self, origin, ts, type, cur=None):
-        """Add a new origin_visit for origin origin at timestamp ts with
-        status 'ongoing'.
+        """Add a new origin_visit for origin origin at timestamp ts.
 
         Args:
             origin: origin concerned by the visit
@@ -477,59 +481,21 @@ class Db(BaseDb):
             + [jsonize(visit_status.metadata)],
         )
 
-    def origin_visit_update(self, origin_id, visit_id, updates, cur=None):
-        """Update origin_visit's status."""
-        cur = self._cursor(cur)
-        update_cols = []
-        values = []
-        where = ["origin.id = origin_visit.origin", "origin.url=%s", "visit=%s"]
-        where_values = [origin_id, visit_id]
-        if "status" in updates:
-            update_cols.append("status=%s")
-            values.append(updates.pop("status"))
-        if "metadata" in updates:
-            update_cols.append("metadata=%s")
-            values.append(jsonize(updates.pop("metadata")))
-        if "snapshot" in updates:
-            update_cols.append("snapshot=%s")
-            values.append(updates.pop("snapshot"))
-        assert not updates, "Unknown fields: %r" % updates
-        query = """UPDATE origin_visit
-                   SET {update_cols}
-                   FROM origin
-                   WHERE {where}""".format(
-            **{"update_cols": ", ".join(update_cols), "where": " AND ".join(where)}
-        )
-        cur.execute(query, (*values, *where_values))
+    def origin_visit_add_with_id(self, origin_visit: OriginVisit, cur=None) -> None:
+        """Insert origin visit when id are already set
 
-    def origin_visit_upsert(self, origin_visit: OriginVisit, cur=None) -> None:
-        # doing an extra query like this is way simpler than trying to join
-        # the origin id in the query below
+        """
         ov = origin_visit
-        origin_id = next(self.origin_id_get_by_url([ov.origin]))
-
+        assert ov.visit is not None
         cur = self._cursor(cur)
-        query = """INSERT INTO origin_visit ({cols}) VALUES ({values})
-                   ON CONFLICT ON CONSTRAINT origin_visit_pkey DO
-                   UPDATE SET {updates}""".format(
-            cols=", ".join(self.origin_visit_get_cols),
-            values=", ".join("%s" for col in self.origin_visit_get_cols),
-            updates=", ".join(
-                "{0}=excluded.{0}".format(col) for col in self.origin_visit_get_cols
-            ),
-        )
-        cur.execute(
-            query,
-            (
-                origin_id,
-                ov.visit,
-                ov.date,
-                ov.type,
-                ov.status,
-                ov.metadata,
-                ov.snapshot,
-            ),
+        origin_visit_cols = ["origin", "visit", "date", "type"]
+        query = """INSERT INTO origin_visit ({cols})
+                   VALUES ((select id from origin where url=%s), {values})
+                   ON CONFLICT (origin, visit) DO NOTHING""".format(
+            cols=", ".join(origin_visit_cols),
+            values=", ".join("%s" for col in origin_visit_cols[1:]),
         )
+        cur.execute(query, (ov.origin, ov.visit, ov.date, ov.type))
 
     origin_visit_get_cols = [
         "origin",
@@ -550,7 +516,18 @@ class Db(BaseDb):
         "ovs.snapshot",
     ]
 
-    def _make_origin_visit_status(self, row: Tuple[Any]) -> Optional[Dict[str, Any]]:
+    origin_visit_status_select_cols = [
+        "o.url AS origin",
+        "ovs.visit",
+        "ovs.date",
+        "ovs.status",
+        "ovs.snapshot",
+        "ovs.metadata",
+    ]
+
+    def _make_origin_visit_status(
+        self, row: Optional[Tuple[Any]]
+    ) -> Optional[Dict[str, Any]]:
         """Make an origin_visit_status dict out of a row
 
         """
@@ -559,25 +536,45 @@ class Db(BaseDb):
         return dict(zip(self.origin_visit_status_cols, row))
 
     def origin_visit_status_get_latest(
-        self, origin: str, visit: int, cur=None
+        self,
+        origin_url: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
+        cur=None,
     ) -> Optional[Dict[str, Any]]:
         """Given an origin visit id, return its latest origin_visit_status
 
         """
-        cols = self.origin_visit_status_cols
         cur = self._cursor(cur)
-        cur.execute(
-            f"SELECT {', '.join(cols)} "
-            f"FROM origin_visit_status ovs "
-            f"INNER JOIN origin o on o.id=ovs.origin "
-            f"WHERE o.url=%s AND ovs.visit=%s"
-            f"ORDER BY ovs.date DESC LIMIT 1",
-            (origin, visit),
-        )
+
+        query_parts = [
+            "SELECT %s" % ", ".join(self.origin_visit_status_select_cols),
+            "FROM origin_visit_status ovs ",
+            "INNER JOIN origin o ON o.id = ovs.origin",
+        ]
+        query_parts.append("WHERE o.url = %s")
+        query_params: List[Any] = [origin_url]
+        query_parts.append("AND ovs.visit = %s")
+        query_params.append(visit)
+
+        if require_snapshot:
+            query_parts.append("AND ovs.snapshot is not null")
+
+        if allowed_statuses:
+            query_parts.append("AND ovs.status IN %s")
+            query_params.append(tuple(allowed_statuses))
+
+        query_parts.append("ORDER BY ovs.date DESC LIMIT 1")
+        query = "\n".join(query_parts)
+
+        cur.execute(query, tuple(query_params))
         row = cur.fetchone()
         return self._make_origin_visit_status(row)
 
-    def origin_visit_get_all(self, origin_id, last_visit=None, limit=None, cur=None):
+    def origin_visit_get_all(
+        self, origin_id, last_visit=None, order="asc", limit=None, cur=None
+    ):
         """Retrieve all visits for origin with id origin_id.
 
         Args:
@@ -588,29 +585,37 @@ class Db(BaseDb):
 
         """
         cur = self._cursor(cur)
+        assert order.lower() in ["asc", "desc"]
 
-        if last_visit:
-            extra_condition = "and ov.visit > %s"
-            args = (origin_id, last_visit, limit)
+        query_parts = [
+            "SELECT DISTINCT ON (ov.visit) %s "
+            % ", ".join(self.origin_visit_select_cols),
+            "FROM origin_visit ov",
+            "INNER JOIN origin o ON o.id = ov.origin",
+            "INNER JOIN origin_visit_status ovs",
+            "ON ov.origin = ovs.origin AND ov.visit = ovs.visit",
+        ]
+        query_parts.append("WHERE o.url = %s")
+        query_params: List[Any] = [origin_id]
+
+        if last_visit is not None:
+            op_comparison = ">" if order == "asc" else "<"
+            query_parts.append(f"and ov.visit {op_comparison} %s")
+            query_params.append(last_visit)
+
+        if order == "asc":
+            query_parts.append("ORDER BY ov.visit ASC, ovs.date DESC")
+        elif order == "desc":
+            query_parts.append("ORDER BY ov.visit DESC, ovs.date DESC")
         else:
-            extra_condition = ""
-            args = (origin_id, limit)
+            assert False
 
-        query = """\
-        SELECT DISTINCT ON (ov.visit) %s
-        FROM origin_visit ov
-        INNER JOIN origin o ON o.id = ov.origin
-        INNER JOIN origin_visit_status ovs
-          ON ov.origin = ovs.origin AND ov.visit = ovs.visit
-        WHERE o.url=%%s %s
-        ORDER BY ov.visit ASC, ovs.date DESC
-        LIMIT %%s""" % (
-            ", ".join(self.origin_visit_select_cols),
-            extra_condition,
-        )
-
-        cur.execute(query, args)
+        if limit is not None:
+            query_parts.append("LIMIT %s")
+            query_params.append(limit)
 
+        query = "\n".join(query_parts)
+        cur.execute(query, tuple(query_params))
         yield from cur
 
     def origin_visit_get(self, origin_id, visit_id, cur=None):
@@ -667,13 +672,19 @@ class Db(BaseDb):
         return bool(cur.fetchone())
 
     def origin_visit_get_latest(
-        self, origin_id: str, allowed_statuses=None, require_snapshot=False, cur=None
+        self,
+        origin_id: str,
+        type: Optional[str],
+        allowed_statuses: Optional[Iterable[str]],
+        require_snapshot: bool,
+        cur=None,
     ):
         """Retrieve the most recent origin_visit of the given origin,
         with optional filters.
 
         Args:
             origin_id: the origin concerned
+            type: Optional visit type to filter on
             allowed_statuses: the visit statuses allowed for the returned visit
             require_snapshot (bool): If True, only a visit with a known
                 snapshot will be returned.
@@ -693,6 +704,10 @@ class Db(BaseDb):
         query_parts.append("WHERE o.url = %s")
         query_params: List[Any] = [origin_id]
 
+        if type is not None:
+            query_parts.append("AND ov.type = %s")
+            query_params.append(type)
+
         if require_snapshot:
             query_parts.append("AND ovs.snapshot is not null")
 
@@ -1071,74 +1086,106 @@ class Db(BaseDb):
     def release_get_random(self, cur=None):
         return self._get_random_row_from_table("release", ["id"], "id", cur)
 
-    origin_metadata_get_cols = [
-        "origin.url",
+    _object_metadata_context_cols = [
+        "origin",
+        "visit",
+        "snapshot",
+        "release",
+        "revision",
+        "path",
+        "directory",
+    ]
+    """The list of context columns for all artifact types."""
+
+    _object_metadata_insert_cols = [
+        "type",
+        "id",
+        "authority_id",
+        "fetcher_id",
+        "discovery_date",
+        "format",
+        "metadata",
+        *_object_metadata_context_cols,
+    ]
+    """List of columns of the object_metadata table, used when writing
+    metadata."""
+
+    _object_metadata_insert_query = f"""
+        INSERT INTO object_metadata
+            ({', '.join(_object_metadata_insert_cols)})
+        VALUES ({', '.join('%s' for _ in _object_metadata_insert_cols)})
+        ON CONFLICT (id, authority_id, discovery_date, fetcher_id)
+        DO NOTHING
+    """
+
+    object_metadata_get_cols = [
+        "id",
         "discovery_date",
         "metadata_authority.type",
         "metadata_authority.url",
         "metadata_fetcher.id",
         "metadata_fetcher.name",
         "metadata_fetcher.version",
+        *_object_metadata_context_cols,
         "format",
         "metadata",
     ]
+    """List of columns of the object_metadata, metadata_authority,
+    and metadata_fetcher tables, used when reading object metadata."""
 
-    def origin_metadata_add(
+    _object_metadata_select_query = f"""
+        SELECT
+            object_metadata.id AS id,
+            {', '.join(object_metadata_get_cols[1:-1])},
+            object_metadata.metadata AS metadata
+        FROM object_metadata
+        INNER JOIN metadata_authority
+            ON (metadata_authority.id=authority_id)
+        INNER JOIN metadata_fetcher ON (metadata_fetcher.id=fetcher_id)
+        WHERE object_metadata.id=%s AND authority_id=%s
+    """
+
+    def object_metadata_add(
         self,
-        origin: str,
+        object_type: str,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
         discovery_date: datetime.datetime,
-        authority: int,
-        fetcher: int,
+        authority_id: int,
+        fetcher_id: int,
         format: str,
         metadata: bytes,
-        cur=None,
-    ) -> None:
-        """ Add an origin_metadata for the origin at ts with provider, tool and
-        metadata.
-
-        Args:
-            origin: the origin's id for which the metadata is added
-            discovery_date: time when the metadata was found
-            authority: the metadata provider identifier
-            fetcher: the tool's identifier used to extract metadata
-            format: the format of the metadata
-            metadata: the metadata retrieved at the time and location
-        """
-        cur = self._cursor(cur)
-        insert = """INSERT INTO origin_metadata (origin_id, discovery_date,
-                    authority_id, fetcher_id, format, metadata)
-                    SELECT id, %s, %s, %s, %s, %s FROM origin WHERE url = %s
-                    ON CONFLICT (origin_id, authority_id, discovery_date, fetcher_id)
-                    DO UPDATE SET
-                        format=EXCLUDED.format,
-                        metadata=EXCLUDED.metadata
-                 """
-        cur.execute(
-            insert, (discovery_date, authority, fetcher, format, metadata, origin),
+        cur,
+    ):
+        query = self._object_metadata_insert_query
+        args: Dict[str, Any] = dict(
+            type=object_type,
+            id=id,
+            authority_id=authority_id,
+            fetcher_id=fetcher_id,
+            discovery_date=discovery_date,
+            format=format,
+            metadata=metadata,
         )
+        for col in self._object_metadata_context_cols:
+            args[col] = context.get(col)
 
-    def origin_metadata_get(
+        params = [args[col] for col in self._object_metadata_insert_cols]
+
+        cur.execute(query, params)
+
+    def object_metadata_get(
         self,
-        origin_url: str,
-        authority: int,
+        object_type: str,
+        id: str,
+        authority_id: int,
         after_time: Optional[datetime.datetime],
         after_fetcher: Optional[int],
-        limit: Optional[int],
-        cur=None,
+        limit: int,
+        cur,
     ):
-        cur = self._cursor(cur)
-        assert self.origin_metadata_get_cols[-1] == "metadata"
-        query_parts = [
-            f"SELECT {', '.join(self.origin_metadata_get_cols[0:-1])}, "
-            f"  origin_metadata.metadata AS metadata "
-            f"FROM origin_metadata "
-            f"INNER JOIN metadata_authority "
-            f"  ON (metadata_authority.id=authority_id) "
-            f"INNER JOIN metadata_fetcher ON (metadata_fetcher.id=fetcher_id) "
-            f"INNER JOIN origin ON (origin.id=origin_metadata.origin_id) "
-            f"WHERE origin.url=%s AND authority_id=%s "
-        ]
-        args = [origin_url, authority]
+        query_parts = [self._object_metadata_select_query]
+        args = [id, authority_id]
 
         if after_fetcher is not None:
             assert after_time
