@@ -25,10 +25,13 @@ from typing import (
     Optional,
     Tuple,
     TypeVar,
+    Union,
 )
 
 import attr
 
+from deprecated import deprecated
+
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
 from swh.model.model import (
     BaseContent,
@@ -45,12 +48,11 @@ from swh.model.model import (
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
 from swh.storage.objstorage import ObjStorage
-from swh.storage.validate import convert_validation_exceptions
 from swh.storage.utils import now
 
-from .exc import StorageArgumentException, HashCollision
-
 from .converters import origin_url_to_sha1
+from .exc import StorageArgumentException, HashCollision
+from .extrinsic_metadata import check_extrinsic_metadata_context, CONTEXT_KEYS
 from .utils import get_partition_bounds_bytes
 from .writer import JournalWriter
 
@@ -137,7 +139,7 @@ class InMemoryStorage:
         self._persons = {}
 
         # {origin_url: {authority: [metadata]}}
-        self._origin_metadata: Dict[
+        self._object_metadata: Dict[
             str,
             Dict[
                 Hashable,
@@ -522,18 +524,19 @@ class InMemoryStorage:
         return random.choice(list(self._revisions))
 
     def release_add(self, releases: Iterable[Release]) -> Dict:
-        releases = [rel for rel in releases if rel.id not in self._releases]
-        self.journal_writer.release_add(releases)
-
-        count = 0
+        to_add = []
         for rel in releases:
+            if rel.id not in self._releases and rel not in to_add:
+                to_add.append(rel)
+        self.journal_writer.release_add(to_add)
+
+        for rel in to_add:
             if rel.author:
                 self._person_add(rel.author)
             self._objects[rel.id].append(("release", rel.id))
             self._releases[rel.id] = rel
-            count += 1
 
-        return {"release:add": count}
+        return {"release:add": len(to_add)}
 
     def release_missing(self, releases):
         yield from (rel for rel in releases if rel not in self._releases)
@@ -578,28 +581,12 @@ class InMemoryStorage:
             return None
 
         visit = self._origin_visit_get_updated(origin_url, visit)
-        snapshot_id = visit.snapshot
+        snapshot_id = visit["snapshot"]
         if snapshot_id:
             return self.snapshot_get(snapshot_id)
         else:
             return None
 
-    def snapshot_get_latest(self, origin, allowed_statuses=None):
-        origin_url = self._get_origin_url(origin)
-        if not origin_url:
-            return
-
-        visit = self.origin_visit_get_latest(
-            origin_url, allowed_statuses=allowed_statuses, require_snapshot=True
-        )
-        if visit and visit["snapshot"]:
-            snapshot = self.snapshot_get(visit["snapshot"])
-            if not snapshot:
-                raise StorageArgumentException(
-                    "last origin visit references an unknown snapshot"
-                )
-            return snapshot
-
     def snapshot_count_branches(self, snapshot_id):
         snapshot = self._snapshots[snapshot_id]
         return collections.Counter(
@@ -613,27 +600,29 @@ class InMemoryStorage:
         snapshot = self._snapshots.get(snapshot_id)
         if snapshot is None:
             return None
-        sorted_branch_names = sorted(snapshot.branches)
+        sorted_branches = sorted(snapshot.branches.items())
+        sorted_branch_names = [k for (k, v) in sorted_branches]
         from_index = bisect.bisect_left(sorted_branch_names, branches_from)
         if target_types:
             next_branch = None
             branches = {}
-            for branch_name in sorted_branch_names[from_index:]:
-                branch = snapshot.branches[branch_name]
-                if branch and branch.target_type.value in target_types:
-                    if len(branches) < branches_count:
-                        branches[branch_name] = branch
-                    else:
-                        next_branch = branch_name
-                        break
+            for (branch_name, branch) in sorted_branches:
+                if branch_name in sorted_branch_names[from_index:]:
+                    if branch and branch.target_type.value in target_types:
+                        if len(branches) < branches_count:
+                            branches[branch_name] = branch
+                        else:
+                            next_branch = branch_name
+                            break
         else:
             # As there is no 'target_types', we can do that much faster
             to_index = from_index + branches_count
-            returned_branch_names = sorted_branch_names[from_index:to_index]
-            branches = {
-                branch_name: snapshot.branches[branch_name]
-                for branch_name in returned_branch_names
-            }
+            returned_branch_names = frozenset(sorted_branch_names[from_index:to_index])
+            branches = dict(
+                (branch_name, branch)
+                for (branch_name, branch) in snapshot.branches.items()
+                if branch_name in returned_branch_names
+            )
             if to_index >= len(sorted_branch_names):
                 next_branch = None
             else:
@@ -752,7 +741,8 @@ class InMemoryStorage:
                     for ov in self._origin_visits[orig["url"]]
                 )
                 for ov in visits:
-                    if ov.snapshot and ov.snapshot in self._snapshots:
+                    snapshot = ov["snapshot"]
+                    if snapshot and snapshot in self._snapshots:
                         filtered_origins.append(orig)
                         break
         else:
@@ -770,12 +760,17 @@ class InMemoryStorage:
             )
         )
 
-    def origin_add(self, origins: Iterable[Origin]) -> List[Dict]:
-        origins = copy.deepcopy(list(origins))
+    def origin_add(self, origins: Iterable[Origin]) -> Dict[str, int]:
+        origins = list(origins)
+        added = 0
         for origin in origins:
-            self.origin_add_one(origin)
-        return [origin.to_dict() for origin in origins]
+            if origin.url not in self._origins:
+                self.origin_add_one(origin)
+                added += 1
 
+        return {"origin:add": added}
+
+    @deprecated("Use origin_add([origin]) instead")
     def origin_add_one(self, origin: Origin) -> str:
         if origin.url not in self._origins:
             self.journal_writer.origin_add([origin])
@@ -808,10 +803,6 @@ class InMemoryStorage:
                     while len(self._origin_visits[origin_url]) < visit.visit:
                         self._origin_visits[origin_url].append(None)
                     self._origin_visits[origin_url][visit.visit - 1] = visit
-                    visit_status_dict = visit.to_dict()
-                    visit_status_dict.pop("type")
-                    visit_status = OriginVisitStatus.from_dict(visit_status_dict)
-                    self._origin_visit_status_add_one(visit_status)
                 else:
                     # visit ids are in the range [1, +inf[
                     visit_id = len(self._origin_visits[origin_url]) + 1
@@ -819,25 +810,31 @@ class InMemoryStorage:
                     self.journal_writer.origin_visit_add([visit])
                     self._origin_visits[origin_url].append(visit)
                     visit_key = (origin_url, visit.visit)
-
-                    visit_status_dict = visit.to_dict()
-                    visit_status_dict.pop("type")
-                    visit_status = OriginVisitStatus.from_dict(visit_status_dict)
-                    self._origin_visit_status_add_one(visit_status)
                     self._objects[visit_key].append(("origin_visit", None))
                 assert visit.visit is not None
+                self._origin_visit_status_add_one(
+                    OriginVisitStatus(
+                        origin=visit.origin,
+                        visit=visit.visit,
+                        date=visit.date,
+                        status="created",
+                        snapshot=None,
+                    )
+                )
                 all_visits.append(visit)
 
         return all_visits
 
     def _origin_visit_status_add_one(self, visit_status: OriginVisitStatus) -> None:
-        """Add an origin visit status without checks.
+        """Add an origin visit status without checks. If already present, do nothing.
 
         """
         self.journal_writer.origin_visit_status_add([visit_status])
         visit_key = (visit_status.origin, visit_status.visit)
         self._origin_visit_statuses.setdefault(visit_key, [])
-        self._origin_visit_statuses[visit_key].append(visit_status)
+        visit_statuses = self._origin_visit_statuses[visit_key]
+        if visit_status not in visit_statuses:
+            visit_statuses.append(visit_status)
 
     def origin_visit_status_add(
         self, visit_statuses: Iterable[OriginVisitStatus],
@@ -851,99 +848,7 @@ class InMemoryStorage:
         for visit_status in visit_statuses:
             self._origin_visit_status_add_one(visit_status)
 
-    def origin_visit_update(
-        self,
-        origin: str,
-        visit_id: int,
-        status: str,
-        metadata: Optional[Dict] = None,
-        snapshot: Optional[bytes] = None,
-        date: Optional[datetime.datetime] = None,
-    ):
-        origin_url = self._get_origin_url(origin)
-        if origin_url is None:
-            raise StorageArgumentException("Unknown origin.")
-
-        try:
-            visit = self._origin_visits[origin_url][visit_id - 1]
-        except IndexError:
-            raise StorageArgumentException("Unknown visit_id for this origin") from None
-
-        updates: Dict[str, Any] = {
-            "status": status,
-        }
-        if metadata and metadata != visit.metadata:
-            updates["metadata"] = metadata
-        if snapshot and snapshot != visit.snapshot:
-            updates["snapshot"] = snapshot
-
-        if updates:
-            with convert_validation_exceptions():
-                updated_visit = OriginVisit.from_dict({**visit.to_dict(), **updates})
-            self.journal_writer.origin_visit_update([updated_visit])
-
-            self._origin_visits[origin_url][visit_id - 1] = updated_visit
-
-            # Retrieve the previous visit status
-            assert visit.visit is not None
-
-            last_visit_status = self._origin_visit_get_updated(origin, visit_id)
-            assert last_visit_status is not None
-
-            with convert_validation_exceptions():
-                visit_status = OriginVisitStatus(
-                    origin=origin_url,
-                    visit=visit_id,
-                    date=date or now(),
-                    status=status,
-                    snapshot=snapshot or last_visit_status.snapshot,
-                    metadata=metadata or last_visit_status.metadata,
-                )
-                self._origin_visit_status_add_one(visit_status)
-
-    def origin_visit_upsert(self, visits: Iterable[OriginVisit]) -> None:
-        for visit in visits:
-            if visit.visit is None:
-                raise StorageArgumentException(f"Missing visit id for visit {visit}")
-
-        self.journal_writer.origin_visit_upsert(visits)
-
-        date = now()
-
-        for visit in visits:
-            assert visit.visit is not None
-            assert visit.visit > 0
-            origin_url = visit.origin
-            origin = self.origin_get({"url": origin_url})
-
-            if not origin:  # Cannot add a visit without an origin
-                raise StorageArgumentException("Unknown origin %s", origin_url)
-
-            if origin_url in self._origins:
-                origin = self._origins[origin_url]
-                # visit ids are in the range [1, +inf[
-                assert visit.visit is not None
-                visit_key = (origin_url, visit.visit)
-
-                with convert_validation_exceptions():
-                    visit_status = OriginVisitStatus(
-                        origin=origin_url,
-                        visit=visit.visit,
-                        date=date,
-                        status=visit.status,
-                        snapshot=visit.snapshot,
-                        metadata=visit.metadata,
-                    )
-
-                while len(self._origin_visits[origin_url]) < visit.visit:
-                    self._origin_visits[origin_url].append(None)
-
-                self._origin_visits[origin_url][visit.visit - 1] = visit
-                self._origin_visit_status_add_one(visit_status)
-
-                self._objects[visit_key].append(("origin_visit", None))
-
-    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> OriginVisit:
+    def _origin_visit_get_updated(self, origin: str, visit_id: int) -> Dict[str, Any]:
         """Merge origin visit and latest origin visit status
 
         """
@@ -953,26 +858,33 @@ class InMemoryStorage:
         visit_key = (origin, visit_id)
 
         visit_update = max(self._origin_visit_statuses[visit_key], key=lambda v: v.date)
-        return OriginVisit.from_dict(
-            {
-                # default to the values in visit
-                **visit.to_dict(),
-                # override with the last update
-                **visit_update.to_dict(),
-                # but keep the date of the creation of the origin visit
-                "date": visit.date,
-            }
-        )
+        return {
+            # default to the values in visit
+            **visit.to_dict(),
+            # override with the last update
+            **visit_update.to_dict(),
+            # but keep the date of the creation of the origin visit
+            "date": visit.date,
+        }
 
     def origin_visit_get(
-        self, origin: str, last_visit: Optional[int] = None, limit: Optional[int] = None
+        self,
+        origin: str,
+        last_visit: Optional[int] = None,
+        limit: Optional[int] = None,
+        order: str = "asc",
     ) -> Iterable[Dict[str, Any]]:
-
+        order = order.lower()
+        assert order in ["asc", "desc"]
         origin_url = self._get_origin_url(origin)
         if origin_url in self._origin_visits:
             visits = self._origin_visits[origin_url]
+            visits = sorted(visits, key=lambda v: v.visit, reverse=(order == "desc"))
             if last_visit is not None:
-                visits = visits[last_visit:]
+                if order == "asc":
+                    visits = [v for v in visits if v.visit > last_visit]
+                else:
+                    visits = [v for v in visits if v.visit < last_visit]
             if limit is not None:
                 visits = visits[:limit]
             for visit in visits:
@@ -982,7 +894,7 @@ class InMemoryStorage:
 
                 visit_update = self._origin_visit_get_updated(origin_url, visit_id)
                 assert visit_update is not None
-                yield visit_update.to_dict()
+                yield visit_update
 
     def origin_visit_find_by_date(
         self, origin: str, visit_date: datetime.datetime
@@ -993,7 +905,7 @@ class InMemoryStorage:
             visit = min(visits, key=lambda v: (abs(v.date - visit_date), -v.visit))
             visit_update = self._origin_visit_get_updated(origin, visit.visit)
             assert visit_update is not None
-            return visit_update.to_dict()
+            return visit_update
         return None
 
     def origin_visit_get_by(self, origin: str, visit: int) -> Optional[Dict[str, Any]]:
@@ -1003,12 +915,13 @@ class InMemoryStorage:
         ):
             visit_update = self._origin_visit_get_updated(origin_url, visit)
             assert visit_update is not None
-            return visit_update.to_dict()
+            return visit_update
         return None
 
     def origin_visit_get_latest(
         self,
         origin: str,
+        type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[Dict[str, Any]]:
@@ -1023,15 +936,41 @@ class InMemoryStorage:
             if visit is not None
         ]
 
+        if type is not None:
+            visits = [visit for visit in visits if visit["type"] == type]
+        if allowed_statuses is not None:
+            visits = [visit for visit in visits if visit["status"] in allowed_statuses]
+        if require_snapshot:
+            visits = [visit for visit in visits if visit["snapshot"]]
+
+        visit = max(visits, key=lambda v: (v["date"], v["visit"]), default=None)
+        if visit is None:
+            return None
+        return visit
+
+    def origin_visit_status_get_latest(
+        self,
+        origin_url: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
+    ) -> Optional[OriginVisitStatus]:
+        ori = self._origins.get(origin_url)
+        if not ori:
+            return None
+
+        visit_key = (origin_url, visit)
+        visits = self._origin_visit_statuses.get(visit_key)
+        if not visits:
+            return None
+
         if allowed_statuses is not None:
             visits = [visit for visit in visits if visit.status in allowed_statuses]
         if require_snapshot:
             visits = [visit for visit in visits if visit.snapshot]
 
-        visit = max(visits, key=lambda v: (v.date, v.visit), default=None)
-        if visit is None:
-            return None
-        return visit.to_dict()
+        visit_status = max(visits, key=lambda v: (v.date, v.visit), default=None)
+        return visit_status
 
     def _select_random_origin_visit_by_type(self, type: str) -> str:
         while True:
@@ -1049,8 +988,11 @@ class InMemoryStorage:
         for visit in random_origin_visits:
             updated_visit = self._origin_visit_get_updated(url, visit.visit)
             assert updated_visit is not None
-            if updated_visit.date > back_in_the_day and updated_visit.status == "full":
-                return updated_visit.to_dict()
+            if (
+                updated_visit["date"] > back_in_the_day
+                and updated_visit["status"] == "full"
+            ):
+                return updated_visit
         else:
             return None
 
@@ -1078,6 +1020,39 @@ class InMemoryStorage:
     def refresh_stat_counters(self):
         pass
 
+    def content_metadata_add(
+        self,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+    ) -> None:
+        self._object_metadata_add(
+            "content",
+            id,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            context,
+        )
+
+    def content_metadata_get(
+        self,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
+        return self._object_metadata_get(
+            "content", id, authority, after, page_token, limit
+        )
+
     def origin_metadata_add(
         self,
         origin_url: str,
@@ -1089,41 +1064,21 @@ class InMemoryStorage:
     ) -> None:
         if not isinstance(origin_url, str):
             raise StorageArgumentException(
-                "origin_id must be str, not %r" % (origin_url,)
+                "origin_url must be str, not %r" % (origin_url,)
             )
-        if not isinstance(metadata, bytes):
-            raise StorageArgumentException(
-                "metadata must be bytes, not %r" % (metadata,)
-            )
-        authority_key = self._metadata_authority_key(authority)
-        if authority_key not in self._metadata_authorities:
-            raise StorageArgumentException(f"Unknown authority {authority}")
-        fetcher_key = self._metadata_fetcher_key(fetcher)
-        if fetcher_key not in self._metadata_fetchers:
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
 
-        origin_metadata_list = self._origin_metadata[origin_url][authority_key]
+        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
 
-        origin_metadata = {
-            "origin_url": origin_url,
-            "discovery_date": discovery_date,
-            "authority": authority_key,
-            "fetcher": fetcher_key,
-            "format": format,
-            "metadata": metadata,
-        }
-
-        for existing_origin_metadata in origin_metadata_list:
-            if (
-                existing_origin_metadata["fetcher"] == fetcher_key
-                and existing_origin_metadata["discovery_date"] == discovery_date
-            ):
-                # Duplicate of an existing one; replace it.
-                existing_origin_metadata.update(origin_metadata)
-                break
-        else:
-            origin_metadata_list.add(origin_metadata)
-        return None
+        self._object_metadata_add(
+            "origin",
+            origin_url,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            context,
+        )
 
     def origin_metadata_get(
         self,
@@ -1136,6 +1091,72 @@ class InMemoryStorage:
         if not isinstance(origin_url, str):
             raise TypeError("origin_url must be str, not %r" % (origin_url,))
 
+        res = self._object_metadata_get(
+            "origin", origin_url, authority, after, page_token, limit
+        )
+        res["results"] = copy.deepcopy(res["results"])
+        for result in res["results"]:
+            result["origin_url"] = result.pop("id")
+
+        return res
+
+    def _object_metadata_add(
+        self,
+        object_type: str,
+        id: str,
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+        context: Dict[str, Union[str, bytes, int]],
+    ) -> None:
+        check_extrinsic_metadata_context(object_type, context)
+        if not isinstance(metadata, bytes):
+            raise StorageArgumentException(
+                "metadata must be bytes, not %r" % (metadata,)
+            )
+        authority_key = self._metadata_authority_key(authority)
+        if authority_key not in self._metadata_authorities:
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        fetcher_key = self._metadata_fetcher_key(fetcher)
+        if fetcher_key not in self._metadata_fetchers:
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
+
+        object_metadata_list = self._object_metadata[id][authority_key]
+
+        object_metadata: Dict[str, Any] = {
+            "id": id,
+            "discovery_date": discovery_date,
+            "authority": authority_key,
+            "fetcher": fetcher_key,
+            "format": format,
+            "metadata": metadata,
+        }
+
+        if CONTEXT_KEYS[object_type]:
+            object_metadata["context"] = context
+
+        for existing_object_metadata in object_metadata_list:
+            if (
+                existing_object_metadata["fetcher"] == fetcher_key
+                and existing_object_metadata["discovery_date"] == discovery_date
+            ):
+                # Duplicate of an existing one; replace it.
+                existing_object_metadata.update(object_metadata)
+                break
+        else:
+            object_metadata_list.add(object_metadata)
+
+    def _object_metadata_get(
+        self,
+        object_type: str,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
         authority_key = self._metadata_authority_key(authority)
 
         if page_token is not None:
@@ -1145,16 +1166,14 @@ class InMemoryStorage:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._origin_metadata[origin_url][authority_key].iter_after(
+            entries = self._object_metadata[id][authority_key].iter_after(
                 (after_time, after_fetcher)
             )
         elif after is not None:
-            entries = self._origin_metadata[origin_url][authority_key].iter_from(
-                (after,)
-            )
+            entries = self._object_metadata[id][authority_key].iter_from((after,))
             entries = (entry for entry in entries if entry["discovery_date"] > after)
         else:
-            entries = iter(self._origin_metadata[origin_url][authority_key])
+            entries = iter(self._object_metadata[id][authority_key])
 
         if limit:
             entries = itertools.islice(entries, 0, limit + 1)
