@@ -5,7 +5,7 @@
 
 import datetime
 
-from typing import Any, Dict, Iterable, List, Optional
+from typing import Any, Dict, Iterable, List, Optional, Union
 
 from swh.core.api import remote_api_endpoint
 from swh.model.model import (
@@ -695,37 +695,6 @@ class StorageInterface:
         """
         ...
 
-    @remote_api_endpoint("snapshot/latest")
-    def snapshot_get_latest(self, origin, allowed_statuses=None):
-        """Get the content, possibly partial, of the latest snapshot for the
-        given origin, optionally only from visits that have one of the given
-        allowed_statuses
-
-        The branches of the snapshot are iterated in the lexicographical
-        order of their names.
-
-        .. warning:: At most 1000 branches contained in the snapshot will be
-            returned for performance reasons. In order to browse the whole
-            set of branches, the method :meth:`snapshot_get_branches`
-            should be used instead.
-
-        Args:
-            origin (str): the origin's URL
-            allowed_statuses (list of str): list of visit statuses considered
-                to find the latest snapshot for the visit. For instance,
-                ``allowed_statuses=['full']`` will only consider visits that
-                have successfully run to completion.
-        Returns:
-            dict: a dict with three keys:
-                * **id**: identifier of the snapshot
-                * **branches**: a dict of branches contained in the snapshot
-                  whose keys are the branches' names.
-                * **next_branch**: the name of the first branch not returned
-                  or :const:`None` if the snapshot has less than 1000
-                  branches.
-        """
-        ...
-
     @remote_api_endpoint("snapshot/count_branches")
     def snapshot_count_branches(self, snapshot_id):
         """Count the number of branches in the snapshot with the given id
@@ -802,6 +771,10 @@ class StorageInterface:
     ) -> None:
         """Add origin visit statuses.
 
+        If there is already a status for the same origin and visit id at the same
+        date, the new one will be either dropped or will replace the existing one
+        (it is unspecified which one of these two behaviors happens).
+
         Args:
             visit_statuses: origin visit statuses to add
 
@@ -810,55 +783,13 @@ class StorageInterface:
         """
         ...
 
-    @remote_api_endpoint("origin/visit/update")
-    def origin_visit_update(
-        self,
-        origin: str,
-        visit_id: int,
-        status: str,
-        metadata: Optional[Dict] = None,
-        snapshot: Optional[bytes] = None,
-        date: Optional[datetime.datetime] = None,
-    ):
-        """Update an origin_visit's status.
-
-        Args:
-            origin (str): visited origin's URL
-            visit_id: Visit's id
-            status: Visit's new status
-            metadata: Data associated to the visit
-            snapshot (sha1_git): identifier of the snapshot to add to
-                the visit
-            date: Update date
-
-        Returns:
-            None
-
-        """
-        ...
-
-    @remote_api_endpoint("origin/visit/upsert")
-    def origin_visit_upsert(self, visits: Iterable[OriginVisit]) -> None:
-        """Add a origin_visits with a specific id and with all its data.
-        If there is already an origin_visit with the same
-        `(origin_id, visit_id)`, overwrites it.
-
-        Args:
-            visits: iterable of dicts with keys:
-
-                - **origin**: dict with keys either `id` or `url`
-                - **visit**: origin visit id
-                - **date**: timestamp of such visit
-                - **status**: Visit's new status
-                - **metadata**: Data associated to the visit
-                - **snapshot**: identifier of the snapshot to add to
-                    the visit
-        """
-        ...
-
     @remote_api_endpoint("origin/visit/get")
     def origin_visit_get(
-        self, origin: str, last_visit: Optional[int] = None, limit: Optional[int] = None
+        self,
+        origin: str,
+        last_visit: Optional[int] = None,
+        limit: Optional[int] = None,
+        order: str = "asc",
     ) -> Iterable[Dict[str, Any]]:
         """Retrieve all the origin's visit's information.
 
@@ -868,6 +799,7 @@ class StorageInterface:
                 Default to None
             limit: Number of results to return from the last visit.
                 Default to None
+            order: Order on visit id fields to list origin visits (default to asc)
 
         Yields:
             List of visits.
@@ -912,15 +844,18 @@ class StorageInterface:
     def origin_visit_get_latest(
         self,
         origin: str,
+        type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[Dict[str, Any]]:
         """Get the latest origin visit for the given origin, optionally
         looking only for those with one of the given allowed_statuses
-        or for those with a known snapshot.
+        or for those with a snapshot.
 
         Args:
             origin: origin URL
+            type: Optional visit type to filter on (e.g git, tar, dsc, svn,
+            hg, npm, pypi, ...)
             allowed_statuses: list of visit statuses considered
                 to find the latest visit. For instance,
                 ``allowed_statuses=['full']`` will only consider visits that
@@ -939,6 +874,35 @@ class StorageInterface:
                 - **metadata**: Data associated to the visit
                 - **snapshot** (Optional[sha1_git]): identifier of the snapshot
                     associated to the visit
+
+        """
+        ...
+
+    @remote_api_endpoint("origin/visit_status/get_latest")
+    def origin_visit_status_get_latest(
+        self,
+        origin_url: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
+    ) -> Optional[OriginVisitStatus]:
+        """Get the latest origin visit status for the given origin visit, optionally
+        looking only for those with one of the given allowed_statuses or with a
+        snapshot.
+
+        Args:
+            origin: origin URL
+
+            allowed_statuses: list of visit statuses considered to find the latest
+                visit. Possible values are {created, ongoing, partial, full}. For
+                instance, ``allowed_statuses=['full']`` will only consider visits that
+                have successfully run to completion.
+            require_snapshot: If True, only a visit with a snapshot
+                will be returned.
+
+        Returns:
+            The OriginVisitStatus matching the criteria
+
         """
         ...
 
@@ -1092,7 +1056,7 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("origin/add_multi")
-    def origin_add(self, origins: Iterable[Origin]) -> List[Dict]:
+    def origin_add(self, origins: Iterable[Origin]) -> Dict[str, int]:
         """Add origins to the storage
 
         Args:
@@ -1103,11 +1067,14 @@ class StorageInterface:
                 - url (bytes): the url the origin points to
 
         Returns:
-            list: given origins as dict updated with their id
+            Summary dict of keys with associated count as values
+
+                origin:add: Count of object actually stored in db
 
         """
         ...
 
+    @deprecated
     @remote_api_endpoint("origin/add")
     def origin_add_one(self, origin: Origin) -> str:
         """Add origin to the storage
@@ -1140,6 +1107,75 @@ class StorageInterface:
         """Recomputes the statistics for `stat_counters`."""
         ...
 
+    @remote_api_endpoint("content/metadata/add")
+    def content_metadata_add(
+        self,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+    ) -> None:
+        """Add a content_metadata for the content at discovery_date,
+        obtained using the `fetcher` from the `authority`.
+
+        The authority and fetcher must be known to the storage before
+        using this endpoint.
+
+        If there is already content metadata for the same content, authority,
+        fetcher, and at the same date; the new one will be either dropped or
+        will replace the existing one
+        (it is unspecified which one of these two behaviors happens).
+
+        Args:
+            discovery_date: when the metadata was fetched.
+            authority: a dict containing keys `type` and `url`.
+            fetcher: a dict containing keys `name` and `version`.
+            format: text field indicating the format of the content of the
+            metadata: blob of raw metadata
+        """
+        ...
+
+    @remote_api_endpoint("content/metadata/get")
+    def content_metadata_get(
+        self,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+    ) -> Dict[str, Any]:
+        """Retrieve list of all content_metadata entries for the id
+
+        Args:
+            id: the content's SWHID
+            authority: a dict containing keys `type` and `url`.
+            after: minimum discovery_date for a result to be returned
+            page_token: opaque token, used to get the next page of results
+            limit: maximum number of results to be returned
+
+        Returns:
+            dict with keys `next_page_token` and `results`.
+            `next_page_token` is an opaque token that is used to get the
+            next page of results, or `None` if there are no more results.
+            `results` is a list of dicts in the format:
+
+            .. code-block: python
+
+                {
+                    'authority': {'type': ..., 'url': ...},
+                    'fetcher': {'name': ..., 'version': ...},
+                    'discovery_date': ...,
+                    'format': '...',
+                    'metadata': b'...',
+                    'context': { ... },
+                }
+
+        """
+        ...
+
     @remote_api_endpoint("origin/metadata/add")
     def origin_metadata_add(
         self,
@@ -1157,7 +1193,9 @@ class StorageInterface:
         using this endpoint.
 
         If there is already origin metadata for the same origin, authority,
-        fetcher, and at the same date, it will be replaced by this one.
+        fetcher, and at the same date; the new one will be either dropped or
+        will replace the existing one
+        (it is unspecified which one of these two behaviors happens).
 
         Args:
             discovery_date: when the metadata was fetched.
@@ -1177,7 +1215,7 @@ class StorageInterface:
         page_token: Optional[bytes] = None,
         limit: int = 1000,
     ) -> Dict[str, Any]:
-        """Retrieve list of all origin_metadata entries for the origin_id
+        """Retrieve list of all origin_metadata entries for the origin_url
 
         Args:
             origin_url: the origin's URL
