@@ -17,7 +17,7 @@ comment on column dbversion.description is 'Release description';
 
 -- latest schema version
 insert into dbversion(version, release, description)
-      values(152, now(), 'Work In Progress');
+      values(158, now(), 'Work In Progress');
 
 -- a SHA1 checksum
 create domain sha1 as bytea check (length(value) = 20);
@@ -37,6 +37,9 @@ create domain unix_path as bytea;
 -- a set of UNIX-like access permissions, as manipulated by, e.g., chmod
 create domain file_perms as int;
 
+-- an SWHID
+create domain swhid as text check (value ~ '^swh:[0-9]+:.*');
+
 
 -- Checksums about actual file content. Note that the content itself is not
 -- stored in the DB, but on external (key-value) storage. A single checksum is
@@ -236,7 +239,8 @@ create table revision
   metadata              jsonb,  -- extra metadata (tarball checksums, extra commit information, etc...)
   object_id             bigserial,
   date_neg_utc_offset   boolean,
-  committer_date_neg_utc_offset boolean
+  committer_date_neg_utc_offset boolean,
+  extra_headers         bytea[][]  -- extra headers (used in hash computation)
 );
 
 comment on table revision is 'A revision represents the state of a source code tree at a specific point in time';
@@ -255,6 +259,7 @@ comment on column revision.committer is 'Committer identity';
 comment on column revision.synthetic is 'True iff revision has been synthesized by Software Heritage';
 comment on column revision.metadata is 'Extra revision metadata';
 comment on column revision.object_id is 'Non-intrinsic, sequential object identifier';
+comment on column revision.extra_headers is 'Extra revision headers; used in revision hash computation';
 
 
 -- either this table or the sha1_git[] column on the revision table
@@ -280,20 +285,13 @@ create table origin_visit
   origin       bigint not null,
   visit        bigint not null,
   date         timestamptz not null,
-  type         text not null,
-  -- remove those when done migrating the schema
-  status       origin_visit_state not null,
-  metadata     jsonb,
-  snapshot     sha1_git
+  type         text not null
 );
 
 comment on column origin_visit.origin is 'Visited origin';
 comment on column origin_visit.visit is 'Sequential visit number for the origin';
 comment on column origin_visit.date is 'Visit timestamp';
 comment on column origin_visit.type is 'Type of loader that did the visit (hg, git, ...)';
-comment on column origin_visit.status is '(Deprecated) Visit status';
-comment on column origin_visit.metadata is '(Deprecated) Optional origin visit metadata';
-comment on column origin_visit.snapshot is '(Deprecated) Optional snapshot of the origin visit. It can be partial.';
 
 
 -- Crawling history of software origin visits by Software Heritage. Each
@@ -427,27 +425,39 @@ comment on column metadata_authority.url is 'Authority''s uri';
 comment on column metadata_authority.metadata is 'Other metadata about authority';
 
 
--- Discovery of metadata during a listing, loading, deposit or external_catalog of an origin
--- also provides a translation to a defined json schema using a translation tool (tool_id)
-create table origin_metadata
+-- Extrinsic metadata on a DAG objects and origins.
+create table object_metadata
 (
-  id             bigserial     not null,  -- PK internal object identifier
-  origin_id      bigint        not null,  -- references origin(id)
-  discovery_date timestamptz   not null,  -- when it was extracted
+  type           text          not null,
+  id             text          not null,
+
+  -- metadata source
   authority_id   bigint        not null,
   fetcher_id     bigint        not null,
-  format         text          not null default 'sword-v2-atom-codemeta-v2-in-json',
-  metadata       bytea         not null
+  discovery_date timestamptz   not null,
+
+  -- metadata itself
+  format         text          not null,
+  metadata       bytea         not null,
+
+  -- context
+  origin         text,
+  visit          bigint,
+  snapshot       swhid,
+  release        swhid,
+  revision       swhid,
+  path           bytea,
+  directory      swhid
 );
 
-comment on table origin_metadata is 'keeps all metadata found concerning an origin';
-comment on column origin_metadata.id is 'the origin_metadata object''s id';
-comment on column origin_metadata.origin_id is 'the origin id for which the metadata was found';
-comment on column origin_metadata.discovery_date is 'the date of retrieval';
-comment on column origin_metadata.authority_id is 'the metadata provider: github, openhub, deposit, etc.';
-comment on column origin_metadata.fetcher_id is 'the tool used for extracting metadata: loaders, crawlers, etc.';
-comment on column origin_metadata.format is 'name of the format of metadata, used by readers to interpret it.';
-comment on column origin_metadata.metadata is 'original metadata in opaque format';
+comment on table object_metadata is 'keeps all metadata found concerning an object';
+comment on column object_metadata.type is 'the type of object (content/directory/revision/release/snapshot/origin) the metadata is on';
+comment on column object_metadata.id is 'the SWHID or origin URL for which the metadata was found';
+comment on column object_metadata.discovery_date is 'the date of retrieval';
+comment on column object_metadata.authority_id is 'the metadata provider: github, openhub, deposit, etc.';
+comment on column object_metadata.fetcher_id is 'the tool used for extracting metadata: loaders, crawlers, etc.';
+comment on column object_metadata.format is 'name of the format of metadata, used by readers to interpret it.';
+comment on column object_metadata.metadata is 'original metadata in opaque format';
 
 
 -- Keep a cache of object counts
