@@ -9,7 +9,8 @@ import itertools
 
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import Any, Dict, Iterable, List, Optional
+from deprecated import deprecated
+from typing import Any, Dict, Iterable, List, Optional, Union
 
 import attr
 import psycopg2
@@ -35,6 +36,10 @@ from swh.storage.validate import VALIDATION_EXCEPTIONS
 from swh.storage.utils import now
 
 from . import converters
+from .extrinsic_metadata import (
+    check_extrinsic_metadata_context,
+    CONTEXT_KEYS,
+)
 from .common import db_transaction_generator, db_transaction
 from .db import Db
 from .exc import StorageArgumentException, StorageDBError, HashCollision
@@ -735,30 +740,6 @@ class Storage:
 
         return None
 
-    @timed
-    @db_transaction(statement_timeout=4000)
-    def snapshot_get_latest(self, origin, allowed_statuses=None, db=None, cur=None):
-        if isinstance(origin, int):
-            origin = self.origin_get({"id": origin}, db=db, cur=cur)
-            if not origin:
-                return
-            origin = origin["url"]
-
-        origin_visit = self.origin_visit_get_latest(
-            origin,
-            allowed_statuses=allowed_statuses,
-            require_snapshot=True,
-            db=db,
-            cur=cur,
-        )
-        if origin_visit and origin_visit["snapshot"]:
-            snapshot = self.snapshot_get(origin_visit["snapshot"], db=db, cur=cur)
-            if not snapshot:
-                raise StorageArgumentException(
-                    "last origin visit references an unknown snapshot"
-                )
-            return snapshot
-
     @timed
     @db_transaction(statement_timeout=2000)
     def snapshot_count_branches(self, snapshot_id, db=None, cur=None):
@@ -841,14 +822,18 @@ class Storage:
                     )
                 visit = attr.evolve(visit, visit=visit_id)
             else:
-                db.origin_visit_upsert(visit)
+                db.origin_visit_add_with_id(visit, cur=cur)
             assert visit.visit is not None
             all_visits.append(visit)
             # Forced to write after for the case when the visit has no id
             self.journal_writer.origin_visit_add([visit])
-            visit_status_dict = visit.to_dict()
-            visit_status_dict.pop("type")
-            visit_status = OriginVisitStatus.from_dict(visit_status_dict)
+            visit_status = OriginVisitStatus(
+                origin=visit.origin,
+                visit=visit.visit,
+                date=visit.date,
+                status="created",
+                snapshot=None,
+            )
             self._origin_visit_status_add(visit_status, db=db, cur=cur)
 
         send_metric("origin_visit:add", count=nb_visits, method_name="origin_visit")
@@ -880,74 +865,21 @@ class Storage:
 
     @timed
     @db_transaction()
-    def origin_visit_update(
+    def origin_visit_status_get_latest(
         self,
-        origin: str,
-        visit_id: int,
-        status: str,
-        metadata: Optional[Dict] = None,
-        snapshot: Optional[bytes] = None,
-        date: Optional[datetime.datetime] = None,
+        origin_url: str,
+        visit: int,
+        allowed_statuses: Optional[List[str]] = None,
+        require_snapshot: bool = False,
         db=None,
         cur=None,
-    ):
-        if not isinstance(origin, str):
-            raise StorageArgumentException(
-                "origin must be a string, not %r" % (origin,)
-            )
-        origin_url = origin
-        visit = db.origin_visit_get(origin_url, visit_id, cur=cur)
-
-        if not visit:
-            raise StorageArgumentException("Invalid visit_id for this origin.")
-
-        visit = dict(zip(db.origin_visit_get_cols, visit))
-
-        updates: Dict[str, Any] = {
-            "status": status,
-        }
-        if metadata and metadata != visit["metadata"]:
-            updates["metadata"] = metadata
-        if snapshot and snapshot != visit["snapshot"]:
-            updates["snapshot"] = snapshot
-
-        if updates:
-            with convert_validation_exceptions():
-                updated_visit = OriginVisit.from_dict({**visit, **updates})
-            self.journal_writer.origin_visit_update([updated_visit])
-
-            # Write updates to origin visit (backward compatibility)
-            db.origin_visit_update(origin, visit_id, updates)
-
-            # Add new origin visit status
-            last_visit_status = self._origin_visit_get_updated(
-                origin, visit_id, db=db, cur=cur
-            )
-            assert last_visit_status is not None
-
-            with convert_validation_exceptions():
-                visit_status = OriginVisitStatus(
-                    origin=origin_url,
-                    visit=visit_id,
-                    date=date or now(),
-                    status=status,
-                    snapshot=snapshot or last_visit_status["snapshot"],
-                    metadata=metadata or last_visit_status["metadata"],
-                )
-                self._origin_visit_status_add(visit_status, db=db, cur=cur)
-
-    def _origin_visit_get_updated(
-        self, origin: str, visit_id: int, db, cur
-    ) -> Optional[Dict[str, Any]]:
-        """Retrieve origin visit and latest origin visit status and merge them
-        into an origin visit.
-
-        """
-        row_visit = db.origin_visit_get(origin, visit_id)
-        if row_visit is None:
+    ) -> Optional[OriginVisitStatus]:
+        row = db.origin_visit_status_get_latest(
+            origin_url, visit, allowed_statuses, require_snapshot, cur=cur
+        )
+        if not row:
             return None
-        visit = dict(zip(db.origin_visit_get_cols, row_visit))
-        return self._origin_visit_apply_update(visit, db=db, cur=cur)
+        return OriginVisitStatus.from_dict(row)
 
     def _origin_visit_apply_update(
         self, visit: Dict[str, Any], db, cur=None
@@ -957,64 +889,19 @@ class Storage:
 
         """
         visit_status = db.origin_visit_status_get_latest(
-            visit["origin"], visit["visit"]
+            visit["origin"], visit["visit"], cur=cur
         )
-        return self._origin_visit_merge(visit, visit_status)
-
-    def _origin_visit_merge(
-        self, visit: Dict[str, Any], visit_status: Dict[str, Any]
-    ) -> Dict[str, Any]:
-        """Merge origin_visit and origin_visit_status together.
-
-        """
-        return OriginVisit.from_dict(
-            {
-                # default to the values in visit
-                **visit,
-                # override with the last update
-                **visit_status,
-                # visit['origin'] is the URL (via a join), while
-                # visit_status['origin'] is only an id.
-                "origin": visit["origin"],
-                # but keep the date of the creation of the origin visit
-                "date": visit["date"],
-            }
-        ).to_dict()
-
-    @timed
-    @db_transaction()
-    def origin_visit_upsert(
-        self, visits: Iterable[OriginVisit], db=None, cur=None
-    ) -> None:
-        visit_statuses = []
-        nb_visits = 0
-        for visit in visits:
-            nb_visits += 1
-            if visit.visit is None:
-                raise StorageArgumentException(f"Missing visit id for visit {visit}")
-            with convert_validation_exceptions():
-                visit_statuses.append(
-                    OriginVisitStatus(
-                        origin=visit.origin,
-                        visit=visit.visit,
-                        date=now(),
-                        status=visit.status,
-                        snapshot=visit.snapshot,
-                        metadata=visit.metadata,
-                    )
-                )
-
-        assert len(visit_statuses) == nb_visits
-
-        # write in journal first
-        self.journal_writer.origin_visit_upsert(visits)
-        self.journal_writer.origin_visit_status_add(visit_statuses)
-
-        # then sync to db
-        for i, visit in enumerate(visits):
-            assert visit.visit is not None
-            db.origin_visit_upsert(visit, cur=cur)
-            db.origin_visit_status_add(visit_statuses[i], cur=cur)
+        return {
+            # default to the values in visit
+            **visit,
+            # override with the last update
+            **visit_status,
+            # visit['origin'] is the URL (via a join), while
+            # visit_status['origin'] is only an id.
+            "origin": visit["origin"],
+            # but keep the date of the creation of the origin visit
+            "date": visit["date"],
+        }
 
     @timed
     @db_transaction_generator(statement_timeout=500)
@@ -1023,11 +910,13 @@ class Storage:
         origin: str,
         last_visit: Optional[int] = None,
         limit: Optional[int] = None,
+        order: str = "asc",
         db=None,
         cur=None,
     ) -> Iterable[Dict[str, Any]]:
+        assert order in ["asc", "desc"]
         lines = db.origin_visit_get_all(
-            origin, last_visit=last_visit, limit=limit, cur=cur
+            origin, last_visit=last_visit, limit=limit, order=order, cur=cur
         )
         for line in lines:
             visit = dict(zip(db.origin_visit_get_cols, line))
@@ -1059,6 +948,7 @@ class Storage:
     def origin_visit_get_latest(
         self,
         origin: str,
+        type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
         db=None,
@@ -1066,6 +956,7 @@ class Storage:
     ) -> Optional[Dict[str, Any]]:
         row = db.origin_visit_get_latest(
             origin,
+            type=type,
             allowed_statuses=allowed_statuses,
             require_snapshot=require_snapshot,
             cur=cur,
@@ -1189,27 +1080,29 @@ class Storage:
         return db.origin_count(url_pattern, regexp, with_visit, cur)
 
     @timed
+    @process_metrics
     @db_transaction()
-    def origin_add(self, origins: Iterable[Origin], db=None, cur=None) -> List[Dict]:
-        origins = list(origins)
-        for origin in origins:
-            self.origin_add_one(origin, db=db, cur=cur)
+    def origin_add(
+        self, origins: Iterable[Origin], db=None, cur=None
+    ) -> Dict[str, int]:
+        urls = [o.url for o in origins]
+        known_origins = set(url for (url,) in db.origin_get_by_url(urls, cur))
+        # use lists here to keep origins sorted; some tests depend on this
+        to_add = [url for url in urls if url not in known_origins]
 
-        return [o.to_dict() for o in origins]
+        self.journal_writer.origin_add([Origin(url=url) for url in to_add])
+        added = 0
+        for url in to_add:
+            if db.origin_add(url, cur):
+                added += 1
+        return {"origin:add": added}
 
+    @deprecated("Use origin_add([origin]) instead")
     @timed
     @db_transaction()
     def origin_add_one(self, origin: Origin, db=None, cur=None) -> str:
-        origin_row = list(db.origin_get_by_url([origin.url], cur))[0]
-        origin_url = dict(zip(db.origin_cols, origin_row))["url"]
-        if origin_url:
-            return origin_url
-
-        self.journal_writer.origin_add([origin])
-
-        url = db.origin_add(origin.url, cur)
-        send_metric("origin:add", count=1, method_name="origin_add_one")
-        return url
+        self.origin_add([origin])
+        return origin.url
 
     @db_transaction(statement_timeout=500)
     def stat_counters(self, db=None, cur=None):
@@ -1236,6 +1129,49 @@ class Storage:
         for key in keys:
             cur.execute("select * from swh_update_counter(%s)", (key,))
 
+    @timed
+    @db_transaction()
+    def content_metadata_add(
+        self,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+        db=None,
+        cur=None,
+    ) -> None:
+        self._object_metadata_add(
+            "content",
+            id,
+            context,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            db,
+            cur,
+        )
+
+    @timed
+    @db_transaction()
+    def content_metadata_get(
+        self,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime] = None,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+        db=None,
+        cur=None,
+    ) -> Dict[str, Any]:
+        return self._object_metadata_get(
+            "content", id, authority, after, page_token, limit, db, cur
+        )
+
     @timed
     @db_transaction()
     def origin_metadata_add(
@@ -1249,29 +1185,20 @@ class Storage:
         db=None,
         cur=None,
     ) -> None:
-        authority_id = db.metadata_authority_get_id(
-            authority["type"], authority["url"], cur
+        context: Dict[str, Union[str, bytes, int]] = {}  # origins have no context
+
+        self._object_metadata_add(
+            "origin",
+            origin_url,
+            context,
+            discovery_date,
+            authority,
+            fetcher,
+            format,
+            metadata,
+            db,
+            cur,
         )
-        if not authority_id:
-            raise StorageArgumentException(f"Unknown authority {authority}")
-        fetcher_id = db.metadata_fetcher_get_id(
-            fetcher["name"], fetcher["version"], cur
-        )
-        if not fetcher_id:
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
-        try:
-            db.origin_metadata_add(
-                origin_url,
-                discovery_date,
-                authority_id,
-                fetcher_id,
-                format,
-                metadata,
-                cur,
-            )
-        except psycopg2.ProgrammingError as e:
-            raise StorageArgumentException(*e.args)
-        send_metric("origin_metadata:add", count=1, method_name="origin_metadata_add")
 
     @timed
     @db_transaction(statement_timeout=500)
@@ -1284,6 +1211,67 @@ class Storage:
         limit: int = 1000,
         db=None,
         cur=None,
+    ) -> Dict[str, Any]:
+        result = self._object_metadata_get(
+            "origin", origin_url, authority, after, page_token, limit, db, cur
+        )
+
+        for res in result["results"]:
+            res.pop("id")
+            res["origin_url"] = origin_url
+
+        return result
+
+    def _object_metadata_add(
+        self,
+        object_type: str,
+        id: str,
+        context: Dict[str, Union[str, bytes, int]],
+        discovery_date: datetime.datetime,
+        authority: Dict[str, Any],
+        fetcher: Dict[str, Any],
+        format: str,
+        metadata: bytes,
+        db,
+        cur,
+    ) -> None:
+        check_extrinsic_metadata_context(object_type, context)
+
+        authority_id = self._get_authority_id(authority, db, cur)
+        fetcher_id = self._get_fetcher_id(fetcher, db, cur)
+        if not isinstance(metadata, bytes):
+            raise StorageArgumentException(
+                "metadata must be bytes, not %r" % (metadata,)
+            )
+
+        db.object_metadata_add(
+            object_type,
+            id,
+            context,
+            discovery_date,
+            authority_id,
+            fetcher_id,
+            format,
+            metadata,
+            cur,
+        )
+
+        send_metric(
+            f"{object_type}_metadata:add",
+            count=1,
+            method_name=f"{object_type}_metadata_add",
+        )
+
+    def _object_metadata_get(
+        self,
+        object_type: str,
+        id: str,
+        authority: Dict[str, str],
+        after: Optional[datetime.datetime],
+        page_token: Optional[bytes],
+        limit: int,
+        db,
+        cur,
     ) -> Dict[str, Any]:
         if page_token:
             (after_time, after_fetcher) = msgpack_loads(page_token)
@@ -1304,28 +1292,39 @@ class Storage:
                 "results": [],
             }
 
-        rows = db.origin_metadata_get(
-            origin_url, authority_id, after_time, after_fetcher, limit + 1, cur
+        rows = db.object_metadata_get(
+            object_type, id, authority_id, after_time, after_fetcher, limit + 1, cur
         )
-        rows = [dict(zip(db.origin_metadata_get_cols, row)) for row in rows]
+        rows = [dict(zip(db.object_metadata_get_cols, row)) for row in rows]
         results = []
         for row in rows:
             row = row.copy()
             row.pop("metadata_fetcher.id")
-            results.append(
-                {
-                    "origin_url": row.pop("origin.url"),
-                    "authority": {
-                        "type": row.pop("metadata_authority.type"),
-                        "url": row.pop("metadata_authority.url"),
-                    },
-                    "fetcher": {
-                        "name": row.pop("metadata_fetcher.name"),
-                        "version": row.pop("metadata_fetcher.version"),
-                    },
-                    **row,
-                }
-            )
+            context = {}
+            for key in CONTEXT_KEYS[object_type]:
+                value = row[key]
+                if value is not None:
+                    context[key] = value
+
+            result = {
+                "id": row["id"],
+                "authority": {
+                    "type": row.pop("metadata_authority.type"),
+                    "url": row.pop("metadata_authority.url"),
+                },
+                "fetcher": {
+                    "name": row.pop("metadata_fetcher.name"),
+                    "version": row.pop("metadata_fetcher.version"),
+                },
+                "discovery_date": row["discovery_date"],
+                "format": row["format"],
+                "metadata": row["metadata"],
+            }
+
+            if CONTEXT_KEYS[object_type]:
+                result["context"] = context
+
+            results.append(result)
 
         if len(results) > limit:
             results.pop()
@@ -1401,3 +1400,19 @@ class Storage:
 
     def flush(self, object_types: Optional[Iterable[str]] = None) -> Dict:
         return {}
+
+    def _get_authority_id(self, authority: Dict[str, Any], db, cur):
+        authority_id = db.metadata_authority_get_id(
+            authority["type"], authority["url"], cur
+        )
+        if not authority_id:
+            raise StorageArgumentException(f"Unknown authority {authority}")
+        return authority_id
+
+    def _get_fetcher_id(self, fetcher: Dict[str, Any], db, cur):
+        fetcher_id = db.metadata_fetcher_get_id(
+            fetcher["name"], fetcher["version"], cur
+        )
+        if not fetcher_id:
+            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
+        return fetcher_id
