@@ -20,11 +20,9 @@ from swh.storage.tests.test_storage import TestStorage as _TestStorage
 
 @pytest.fixture
 def app_server():
-    storage_config = {
-        "cls": "memory",
-        "journal_writer": {"cls": "memory",},
-    }
-    server.storage = swh.storage.get_storage(**storage_config)
+    server.storage = swh.storage.get_storage(
+        cls="memory", journal_writer={"cls": "memory"}
+    )
     yield server
 
 
