@@ -53,14 +53,8 @@ def test_storage_direct_writer(kafka_prefix: str, kafka_server, consumer: Consum
             for obj in objs:
                 assert isinstance(obj, OriginVisit)
                 storage.origin_add_one(Origin(url=obj.origin))
-                visit = method([obj])[0]
+                method([obj])
                 expected_messages += 1 + 1  # 1 visit + 1 visit status
-
-                obj_d = obj.to_dict()
-                for k in ("visit", "origin", "date", "type"):
-                    del obj_d[k]
-                storage.origin_visit_update(obj.origin, visit.visit, **obj_d)
-                expected_messages += 1 + 1  # 1 visit update + 1 visit status
         else:
             assert False, obj_type
 
